@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: align.cpp 1302 2008-07-18 22:49:53Z bengardner $
+ * $Id: align.cpp 1471 2009-01-28 01:45:08Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -212,7 +212,7 @@ void quick_align_again(void)
    chunk_t *tmp;
    AlignStack as;
 
-   LOG_FMT(LALAGAIN, "%s\n", __func__);
+   LOG_FMT(LALAGAIN, "%s:\n", __func__);
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
       if ((pc->align.next != NULL) && (pc->flags & PCF_ALIGN_START))
@@ -224,10 +224,10 @@ void quick_align_again(void)
          as.m_gap         = pc->align.gap;
 
          LOG_FMT(LALAGAIN, "   [%.*s:%d]", pc->len, pc->str, pc->orig_line);
-         as.Add(pc);
+         as.Add(pc->align.start);
          for (tmp = pc->align.next; tmp != NULL; tmp = tmp->align.next)
          {
-            as.Add(tmp);
+            as.Add(tmp->align.start);
             LOG_FMT(LALAGAIN, " => [%.*s:%d]", tmp->len, tmp->str, tmp->orig_line);
          }
          LOG_FMT(LALAGAIN, "\n");
@@ -764,7 +764,7 @@ static void align_same_func_call_params()
    chunk_t    *chunks[16];
    AlignStack as[16];
    AlignStack fcn_as;
-   int        max_idx = 0;
+   int        max_idx = -1;
    int        cur_as;
    int        idx;
    const char *add_str = NULL;
@@ -886,7 +886,15 @@ static void align_func_proto(int span)
                ((pc->type == CT_FUNC_DEF) &&
                 cpd.settings[UO_align_single_line_func].b))
       {
-         as.Add(pc);
+         if ((pc->parent_type == CT_OPERATOR) &&
+             cpd.settings[UO_align_on_operator].b)
+         {
+            as.Add(chunk_get_prev_ncnl(pc));
+         }
+         else
+         {
+            as.Add(pc);
+         }
          look_bro = (pc->type == CT_FUNC_DEF) &&
                     cpd.settings[UO_align_single_line_brace].b;
       }
@@ -1062,7 +1070,8 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
       }
 
       /* If this is a variable def, update the max_col */
-      if (((pc->flags & align_mask) == PCF_VAR_1ST) &&
+      if ((pc->type != CT_FUNC_CLASS) &&
+          ((pc->flags & align_mask) == PCF_VAR_1ST) &&
           ((pc->level == (start->level + 1)) ||
            (pc->level == 0)))
       {
@@ -1084,6 +1093,14 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
          }
          did_this_line = true;
       }
+      if (pc->type == CT_BIT_COLON)
+      {
+         if (!did_this_line)
+         {
+            as_bc.Add(pc);
+            did_this_line = true;
+         }
+      }
       pc = chunk_get_next(pc);
    }
 
