@@ -6,7 +6,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: align_stack.h 1158 2008-04-12 20:51:38Z bengardner $
+ * $Id: align_stack.h 1325 2008-08-16 17:28:50Z bengardner $
  */
 
 #include "ChunkStack.h"
@@ -21,9 +21,8 @@ public:
       SS_DANGLE   // include prev * after add
    };
 
-   ChunkStack m_aligned;
-   ChunkStack m_skipped;
-   ChunkStack m_scratch;
+   ChunkStack m_aligned;   /* contains the token that is aligned */
+   ChunkStack m_skipped;   /* contains the tokens sent to Add() */
    int        m_max_col;
    int        m_min_col;
    int        m_span;
@@ -36,9 +35,14 @@ public:
    StarStyle  m_amp_style;
 
 
-   AlignStack()
+   AlignStack() :
+      m_max_col(0), m_min_col(0), m_span(0), m_thresh(0), m_seqnum(0),
+      m_nl_seqnum(0), m_gap(0), m_right_align(false),
+      m_star_style(SS_IGNORE), m_amp_style(SS_IGNORE),
+      m_last_added(0)
    {
    }
+
    ~AlignStack()
    {
    }
@@ -52,4 +56,5 @@ public:
 protected:
    int m_last_added; /* 0=none, 1=aligned, 2=skipped */
    void ReAddSkipped();
+   ChunkStack m_scratch; /* used in ReAddSkipped() */
 };
