@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: braces.cpp 1253 2008-06-18 02:22:42Z bengardner $
+ * $Id: braces.cpp 1368 2008-09-12 01:45:00Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -60,7 +60,7 @@ void do_braces(void)
 
       /* Detect empty bodies */
       tmp = chunk_get_next_ncnl(pc);
-      if (tmp->type == CT_BRACE_CLOSE)
+      if ((tmp != NULL) && (tmp->type == CT_BRACE_CLOSE))
       {
          br_open->flags |= PCF_EMPTY_BODY;
          tmp->flags     |= PCF_EMPTY_BODY;
@@ -444,7 +444,8 @@ void add_long_closebrace_comment(void)
    pc = chunk_get_head();
    while ((pc = chunk_get_next_ncnl(pc)) != NULL)
    {
-      if (pc->type == CT_FUNC_DEF)
+      if ((pc->type == CT_FUNC_DEF) ||
+          (pc->type == CT_OC_MSG_DECL))
       {
          fcn_pc = pc;
       }
@@ -488,7 +489,8 @@ void add_long_closebrace_comment(void)
                   nl_min = cpd.settings[UO_mod_add_long_switch_closebrace_comment].n;
                   tag_pc = sw_pc;
                }
-               else if (br_open->parent_type == CT_FUNC_DEF)
+               else if ((br_open->parent_type == CT_FUNC_DEF) ||
+                        (br_open->parent_type == CT_OC_MSG_DECL))
                {
                   nl_min = cpd.settings[UO_mod_add_long_function_closebrace_comment].n;
                   tag_pc = fcn_pc;
@@ -496,10 +498,12 @@ void add_long_closebrace_comment(void)
 
                if ((nl_min > 0) && (nl_count >= nl_min) && (tag_pc != NULL))
                {
-                  /* TODO: determine the added comment style */
+                  /* determine the added comment style */
+                  c_token_t style = (cpd.lang_flags & (LANG_CPP | LANG_CS)) ?
+                                    CT_COMMENT_CPP : CT_COMMENT;
 
                   /* Add a comment after the close brace */
-                  insert_comment_after(br_close, CT_COMMENT,
+                  insert_comment_after(br_close, style,
                                        tag_pc->len, tag_pc->str);
                }
             }
