@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: combine.cpp 1294 2008-07-18 01:04:49Z bengardner $
+ * $Id: combine.cpp 1487 2009-02-27 02:25:45Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -40,7 +40,10 @@ static void mark_lvalue(chunk_t *pc);
 static void mark_template_func(chunk_t *pc, chunk_t *pc_next);
 static void mark_exec_sql(chunk_t *pc);
 static void handle_oc_class(chunk_t *pc);
-static void handle_oc_message(chunk_t *pc);
+static void handle_oc_message_decl(chunk_t *pc);
+static void handle_oc_message_send(chunk_t *pc);
+static void handle_cs_square_stmt(chunk_t *pc);
+static void handle_cs_property(chunk_t *pc);
 static void handle_template(chunk_t *pc);
 
 
@@ -279,15 +282,66 @@ void fix_symbols(void)
          }
       } /* paren open + cast/align/delegate */
 
+      if (pc->type == CT_INVARIANT)
+      {
+         if (next->type == CT_PAREN_OPEN)
+         {
+            next->parent_type = pc->type;
+            tmp = chunk_get_next(next);
+            while (tmp != NULL)
+            {
+               if (tmp->type == CT_PAREN_CLOSE)
+               {
+                  tmp->parent_type = pc->type;
+                  break;
+               }
+               make_type(tmp);
+               tmp = chunk_get_next(tmp);
+            }
+         }
+         else
+         {
+            pc->type = CT_QUALIFIER;
+         }
+      }
+
       /* Objective C stuff */
       if (cpd.lang_flags & LANG_OC)
       {
          /* Check for message declarations */
-         if ((pc->flags & PCF_STMT_START) &&
-             (chunk_is_str(pc, "-", 1) || chunk_is_str(pc, "+", 1)) &&
-             chunk_is_str(next, "(", 1))
+         if (pc->flags & PCF_STMT_START)
          {
-            handle_oc_message(pc);
+            if ((chunk_is_str(pc, "-", 1) || chunk_is_str(pc, "+", 1)) &&
+                chunk_is_str(next, "(", 1))
+            {
+               handle_oc_message_decl(pc);
+            }
+         }
+         if (pc->flags & PCF_EXPR_START)
+         {
+            if (pc->type == CT_SQUARE_OPEN)
+            {
+               handle_oc_message_send(pc);
+            }
+         }
+      }
+
+      /* C# stuff */
+      if (cpd.lang_flags & LANG_CS)
+      {
+         /* '[assembly: xxx]' stuff */
+         if ((pc->flags & PCF_EXPR_START) &&
+             (pc->type == CT_SQUARE_OPEN))
+         {
+            handle_cs_square_stmt(pc);
+         }
+
+         if ((next != NULL) && (next->type == CT_BRACE_OPEN) &&
+             (next->parent_type == CT_NONE) &&
+             ((pc->type == CT_SQUARE_CLOSE) ||
+              (pc->type == CT_WORD)))
+         {
+            handle_cs_property(next);
          }
       }
 
@@ -344,10 +398,7 @@ void fix_symbols(void)
       {
          if (prev->type != CT_TYPEDEF)
          {
-            if ((pc->flags & PCF_IN_TYPEDEF) == 0)
-            {
-               fix_enum_struct_union(pc);
-            }
+            fix_enum_struct_union(pc);
          }
       }
 
@@ -489,6 +540,7 @@ void fix_symbols(void)
       /* Mark function parens and braces */
       if ((pc->type == CT_FUNC_DEF) ||
           (pc->type == CT_FUNC_CALL) ||
+          (pc->type == CT_FUNC_CALL_USER) ||
           (pc->type == CT_FUNC_PROTO))
       {
          tmp = next;
@@ -537,7 +589,8 @@ void fix_symbols(void)
       /* Mark the braces in: "for_each_entry(xxx) { }" */
       if ((pc->type == CT_BRACE_OPEN) &&
           (prev->type == CT_FPAREN_CLOSE) &&
-          (prev->parent_type == CT_FUNC_CALL) &&
+          ((prev->parent_type == CT_FUNC_CALL) ||
+           (prev->parent_type == CT_FUNC_CALL_USER)) &&
           ((pc->flags & PCF_IN_CONST_ARGS) == 0))
       {
          set_paren_parent(pc, CT_FUNC_CALL);
@@ -598,6 +651,8 @@ void fix_symbols(void)
               (next->type == CT_TYPE) ||
               (next->type == CT_STRUCT) ||
               (next->type == CT_QUALIFIER) ||
+              (next->type == CT_MEMBER) ||
+              (next->type == CT_DC_MEMBER) ||
               (next->type == CT_ENUM) ||
               (next->type == CT_UNION)) &&
              (prev->type != CT_SIZEOF) &&
@@ -690,13 +745,20 @@ void fix_symbols(void)
          {
             pc->type = CT_DEREF;
          }
-         else if ((prev->type == CT_WORD) && chunk_ends_type(prev))
+         else if (((prev->type == CT_WORD) && chunk_ends_type(prev)) ||
+                  (prev->type == CT_DC_MEMBER))
          {
             pc->type = CT_PTR_TYPE;
          }
          else
          {
-            pc->type = (prev->type == CT_ARITH) ? CT_DEREF : CT_ARITH;
+            /* most PCF_PUNCTUATOR chunks except a paren close would make this
+             * a deref. A paren close may end a cast.
+             */
+            pc->type = ((prev->flags & PCF_PUNCTUATOR) &&
+                        !chunk_is_paren_close(prev) &&
+                        (prev->type != CT_SQUARE_CLOSE) &&
+                        (prev->type != CT_DC_MEMBER)) ? CT_DEREF : CT_ARITH;
          }
       }
 
@@ -712,9 +774,10 @@ void fix_symbols(void)
             if (prev->type == CT_WORD)
             {
                tmp = chunk_get_prev_ncnl(prev);
-               if (chunk_is_semicolon(tmp) ||
-                   (tmp->type == CT_BRACE_OPEN) ||
-                   (tmp->type == CT_QUALIFIER))
+               if ((tmp != NULL) &&
+                   (chunk_is_semicolon(tmp) ||
+                    (tmp->type == CT_BRACE_OPEN) ||
+                    (tmp->type == CT_QUALIFIER)))
                {
                   prev->type   = CT_TYPE;
                   pc->type     = CT_ADDR;
@@ -917,6 +980,7 @@ static void mark_function_type(chunk_t *pc)
          {
             if ((tmp->type == CT_FUNCTION) ||
                 (tmp->type == CT_FUNC_CALL) ||
+                (tmp->type == CT_FUNC_CALL_USER) ||
                 (tmp->type == CT_FUNC_DEF) ||
                 (tmp->type == CT_FUNC_PROTO))
             {
@@ -960,7 +1024,6 @@ static chunk_t *process_return(chunk_t *pc)
    chunk_t *cpar;
    chunk_t chunk;
 
-
    /* grab next and bail if it is a semicolon */
    next = chunk_get_next_ncnl(pc);
    if ((next == NULL) || chunk_is_semicolon(next))
@@ -1023,12 +1086,13 @@ static chunk_t *process_return(chunk_t *pc)
    semi = next;
    while ((semi = chunk_get_next(semi)) != NULL)
    {
-      if (chunk_is_semicolon(semi) && (next->level == semi->level))
+      if ((chunk_is_semicolon(semi) && (pc->level == semi->level)) ||
+          (semi->level < pc->level))
       {
          break;
       }
    }
-   if (semi != NULL)
+   if (chunk_is_semicolon(semi) && (pc->level == semi->level))
    {
       /* add the parens */
       memset(&chunk, 0, sizeof(chunk));
@@ -1246,6 +1310,7 @@ static void fix_casts(chunk_t *start)
                (pc->type != CT_STRING) &&
                (pc->type != CT_SIZEOF) &&
                (pc->type != CT_FUNC_CALL) &&
+               (pc->type != CT_FUNC_CALL_USER) &&
                (pc->type != CT_FUNCTION))
       {
          LOG_FMT(LCASTS, " -- not a cast - followed by '%.*s' %s\n",
@@ -1657,6 +1722,19 @@ void combine_labels(void)
                else
                {
                   next->type = CT_BIT_COLON;
+
+                  tmp = chunk_get_next(next);
+                  while ((tmp = chunk_get_next(tmp)) != NULL)
+                  {
+                     if (tmp->type == CT_SEMICOLON)
+                     {
+                        break;
+                     }
+                     if (tmp->type == CT_COLON)
+                     {
+                        tmp->type = CT_BIT_COLON;
+                     }
+                  }
                }
             }
             else if (nextprev->type == CT_FPAREN_CLOSE)
@@ -1725,7 +1803,8 @@ static void mark_variable_stack(ChunkStack& cs, log_sev_t sev)
       while ((word_type = cs.Pop()) != NULL)
       {
          LOG_FMT(LFCNP, " <%.*s>", word_type->len, word_type->str);
-         word_type->type = CT_TYPE;
+         word_type->type   = CT_TYPE;
+         word_type->flags |= PCF_VAR_TYPE;
       }
 
       LOG_FMT(LFCNP, " [%.*s]\n", var_name->len, var_name->str);
@@ -1752,18 +1831,26 @@ static void fix_fcn_def_params(chunk_t *start)
 
    ChunkStack cs;
 
+   int level = start->level + 1;
+
    chunk_t *pc = start;
    while ((pc = chunk_get_next_ncnl(pc)) != NULL)
    {
-      LOG_FMT(LFCNP, "%s: looking at %.*s on line %d, level %d\n", __func__, pc->len, pc->str, pc->orig_line, pc->level);
-
       if (((start->len == 1) && (*start->str == ')')) ||
-          (pc->level <= start->level))
+          (pc->level < level))
       {
          LOG_FMT(LFCNP, "%s: bailed on %.*s on line %d\n", __func__, pc->len, pc->str, pc->orig_line);
          break;
       }
 
+      LOG_FMT(LFCNP, "%s: %s %.*s on line %d, level %d\n", __func__,
+              (pc->level > level) ? "skipping" : "looking at",
+              pc->len, pc->str, pc->orig_line, pc->level);
+
+      if (pc->level > level)
+      {
+         continue;
+      }
       if (chunk_is_star(pc))
       {
          pc->type = CT_PTR_TYPE;
@@ -1896,6 +1983,7 @@ static chunk_t *fix_var_def(chunk_t *start)
    {
       tmp_pc = cs.Get(idx)->m_pc;
       make_type(tmp_pc);
+      tmp_pc->flags |= PCF_VAR_TYPE;
       LOG_FMT(LFVD2, " %.*s[%s]", tmp_pc->len, tmp_pc->str, get_token_name(tmp_pc->type));
    }
    LOG_FMT(LFVD2, "\n");
@@ -1953,7 +2041,6 @@ static chunk_t *mark_variable_definition(chunk_t *start)
       return(NULL);
    }
 
-
    LOG_FMT(LVARDEF, "%s: line %d, col %d '%.*s' type %s\n",
            __func__,
            pc->orig_line, pc->orig_col, pc->len, pc->str,
@@ -2416,7 +2503,8 @@ static void mark_function(chunk_t *pc)
 
       if (isa_def && (prev != NULL) &&
           (chunk_is_paren_close(prev) ||
-           (prev->type == CT_ASSIGN)))
+           (prev->type == CT_ASSIGN) ||
+           (prev->type == CT_RETURN)))
       {
          LOG_FMT(LFCN, " -- overriding DEF due to %.*s [%s]\n",
                  prev->len, prev->str, get_token_name(prev->type));
@@ -2556,19 +2644,26 @@ static void mark_function(chunk_t *pc)
       }
       else if (pc->brace_level > 0)
       {
-         /* Do a check to see if the level is right */
-         prev = chunk_get_prev_ncnl(pc);
-         if (!chunk_is_str(prev, "*", 1) && !chunk_is_str(prev, "&", 1))
+         chunk_t *br_open = chunk_get_prev_type(pc, CT_BRACE_OPEN, pc->brace_level - 1);
+
+         if ((br_open != NULL) &&
+             (br_open->parent_type != CT_EXTERN) &&
+             (br_open->parent_type != CT_NAMESPACE))
          {
-            chunk_t *p_op = chunk_get_prev_type(pc, CT_BRACE_OPEN, pc->brace_level - 1);
-            if ((p_op != NULL) &&
-                (p_op->parent_type != CT_CLASS) &&
-                (p_op->parent_type != CT_STRUCT) &&
-                (p_op->parent_type != CT_NAMESPACE))
+            /* Do a check to see if the level is right */
+            prev = chunk_get_prev_ncnl(pc);
+            if (!chunk_is_str(prev, "*", 1) && !chunk_is_str(prev, "&", 1))
             {
-               pc->type = CT_FUNC_CTOR_VAR;
-               LOG_FMT(LFCN, "  4) Marked [%.*s] as FUNC_CTOR_VAR on line %d col %d\n",
-                       pc->len, pc->str, pc->orig_line, pc->orig_col);
+               chunk_t *p_op = chunk_get_prev_type(pc, CT_BRACE_OPEN, pc->brace_level - 1);
+               if ((p_op != NULL) &&
+                   (p_op->parent_type != CT_CLASS) &&
+                   (p_op->parent_type != CT_STRUCT) &&
+                   (p_op->parent_type != CT_NAMESPACE))
+               {
+                  pc->type = CT_FUNC_CTOR_VAR;
+                  LOG_FMT(LFCN, "  4) Marked [%.*s] as FUNC_CTOR_VAR on line %d col %d\n",
+                          pc->len, pc->str, pc->orig_line, pc->orig_col);
+               }
             }
          }
       }
@@ -3084,6 +3179,7 @@ static void mark_template_func(chunk_t *pc, chunk_t *pc_next)
       {
          // its a type!
          pc->type      = CT_TYPE;
+         pc->flags    |= PCF_VAR_TYPE;
          after->flags |= PCF_VAR_DEF;
       }
    }
@@ -3202,15 +3298,24 @@ static void handle_oc_class(chunk_t *pc)
 
    while ((tmp = chunk_get_next(tmp)) != NULL)
    {
-      if ((tmp->type == CT_SEMICOLON) ||
-          (tmp->type == CT_BRACE_OPEN))
+      if (tmp->type == CT_OC_END)
+      {
+         break;
+      }
+      if (tmp->type == CT_BRACE_OPEN)
       {
          tmp->parent_type = CT_OC_CLASS;
          break;
       }
       else if (tmp->type == CT_COLON)
       {
-         tmp->type = CT_CLASS_COLON;
+         tmp->type        = CT_CLASS_COLON;
+         tmp->parent_type = CT_OC_CLASS;
+      }
+      else if (chunk_is_str(tmp, "-", 1))
+      {
+         tmp->flags |= PCF_STMT_START;
+         break;
       }
    }
 
@@ -3236,12 +3341,12 @@ static void handle_oc_class(chunk_t *pc)
  * ARGS is ': (type) name [name]'
  * -(void) foo: (int) arg: {  }
  */
-static void handle_oc_message(chunk_t *pc)
+static void handle_oc_message_decl(chunk_t *pc)
 {
-   chunk_t *tmp;
-   bool    in_paren  = false;
-   int     paren_cnt = 0;
-   c_token_t pt;
+   chunk_t   *tmp;
+   bool      in_paren  = false;
+   int       paren_cnt = 0;
+   c_token_t pt = CT_OC_MSG_SPEC;
 
    /* Figure out if this is a spec or decl */
    tmp = pc;
@@ -3314,3 +3419,156 @@ static void handle_oc_message(chunk_t *pc)
    }
 }
 
+/**
+ * Process an ObjC message send statement:
+ * [ server setStringValue : @"" ] ;
+ *
+ * Just find the matching ']' and ';' and mark the colon.
+ *
+ * @param os points to the open square '['
+ */
+static void handle_oc_message_send(chunk_t *os)
+{
+   chunk_t *tmp;
+   chunk_t *cs = chunk_get_next(os);
+
+   while ((cs != NULL) && (cs->level > os->level))
+   {
+      cs = chunk_get_next(cs);
+   }
+
+   if ((cs == NULL) || (cs->type != CT_SQUARE_CLOSE))
+   {
+      return;
+   }
+
+   tmp = chunk_get_next_ncnl(cs);
+   if (chunk_is_semicolon(tmp))
+   {
+      tmp->parent_type = CT_OC_MSG;
+   }
+
+   os->parent_type  = CT_OC_MSG;
+   cs->parent_type  = CT_OC_MSG;
+
+   for (tmp = chunk_get_next(os); tmp != cs; tmp = chunk_get_next(tmp))
+   {
+      tmp->parent_type = CT_OC_MSG;
+      if (tmp->type == CT_COLON)
+      {
+         tmp->type = CT_OC_COLON;
+      }
+   }
+}
+
+/**
+ * Process an C# [] thingy:
+ *    [assembly: xxx]
+ *    [AttributeUsage()]
+ *    [@X]
+ *
+ * Set the next chunk to a statement start after the close ']'
+ *
+ * @param os points to the open square '['
+ */
+static void handle_cs_square_stmt(chunk_t *os)
+{
+   chunk_t *tmp;
+   chunk_t *cs = chunk_get_next(os);
+
+   while ((cs != NULL) && (cs->level > os->level))
+   {
+      cs = chunk_get_next(cs);
+   }
+
+   if ((cs == NULL) || (cs->type != CT_SQUARE_CLOSE))
+   {
+      return;
+   }
+
+   os->parent_type = CT_CS_SQ_STMT;
+   cs->parent_type = CT_CS_SQ_STMT;
+
+   for (tmp = chunk_get_next(os); tmp != cs; tmp = chunk_get_next(tmp))
+   {
+      tmp->parent_type = CT_CS_SQ_STMT;
+      if (tmp->type == CT_COLON)
+      {
+         tmp->type = CT_CS_SQ_COLON;
+      }
+   }
+
+   tmp = chunk_get_next_ncnl(cs);
+   if (tmp != NULL)
+   {
+      tmp->flags |= PCF_STMT_START | PCF_EXPR_START;
+   }
+}
+
+/**
+ * We are on a brace open that is preceeded by a word or square close.
+ * Set the brace parent to CT_CS_PROPERTY and find the first item in the
+ * property and set its parent, too.
+ */
+static void handle_cs_property(chunk_t *bro)
+{
+   chunk_t *pc;
+   bool    did_prop = false;
+
+   set_paren_parent(bro, CT_CS_PROPERTY);
+
+   pc = bro;
+   while ((pc = chunk_get_prev_ncnl(pc)) != NULL)
+   {
+      if (pc->level == bro->level)
+      {
+         if (!did_prop && ((pc->type == CT_WORD) || (pc->type == CT_THIS)))
+         {
+            pc->type = CT_CS_PROPERTY;
+            did_prop = true;
+         }
+         else
+         {
+            pc->parent_type = CT_CS_PROPERTY;
+            make_type(pc);
+         }
+         if (pc->flags & PCF_STMT_START)
+         {
+            break;
+         }
+      }
+   }
+}
+
+/**
+ * Remove 'return;' that appears as the last statement in a function
+ */
+void remove_extra_returns()
+{
+   chunk_t *pc;
+   chunk_t *semi;
+   chunk_t *cl_br;
+
+   pc = chunk_get_head();
+   while (pc != NULL)
+   {
+      if ((pc->type == CT_RETURN) && (pc->flags & PCF_IN_PREPROC) == 0)
+      {
+         semi = chunk_get_next_ncnl(pc);
+         cl_br = chunk_get_next_ncnl(semi);
+
+         if ((semi != NULL) && (semi->type == CT_SEMICOLON) &&
+             (cl_br != NULL) && (cl_br->type == CT_BRACE_CLOSE) &&
+             ((cl_br->parent_type == CT_FUNC_DEF) ||
+              (cl_br->parent_type == CT_FUNC_CLASS)))
+         {
+            LOG_FMT(LRMRETURN, "Removed 'return;' on line %d\n", pc->orig_line);
+            chunk_del(pc);
+            chunk_del(semi);
+            pc = cl_br;
+         }
+      }
+
+      pc = chunk_get_next(pc);
+   }
+}
