@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: indent.cpp 1297 2008-07-18 01:32:15Z bengardner $
+ * $Id: indent.cpp 1481 2009-02-21 18:44:54Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -180,6 +180,8 @@ void align_to_column(chunk_t *pc, int column)
          {
             pc->column = min_col;
          }
+         LOG_FMT(LINDLINED, "   set column of '%.*s' to %d (orig %d)\n",
+                 pc->len, pc->str, pc->column, pc->orig_col);
       }
    } while ((pc != NULL) && (pc->nl_count == 0));
 }
@@ -239,6 +241,8 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
          {
             pc->column = min_col;
          }
+         LOG_FMT(LINDLINED, "   set column of '%.*s' to %d (orig %d)\n",
+                 pc->len, pc->str, pc->column, pc->orig_col);
       }
    } while ((pc != NULL) && (pc->nl_count == 0));
 }
@@ -270,6 +274,7 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
       frm.pse[frm.pse_tos].ref        = ++ref;
       frm.pse[frm.pse_tos].in_preproc = (pc->flags & PCF_IN_PREPROC) != 0;
       frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab;
+      frm.pse[frm.pse_tos].non_vardef = false;
    }
 }
 
@@ -597,9 +602,10 @@ void indent_text(void)
                indent_pse_pop(frm, pc);
             }
 
-            /* End any CPP class colon crap */
+            /* End any CPP/ObjC class colon stuff */
             if ((frm.pse[frm.pse_tos].type == CT_CLASS_COLON) &&
                 ((pc->type == CT_BRACE_OPEN) ||
+                 (pc->type == CT_OC_END) ||
                  chunk_is_semicolon(pc)))
             {
                indent_pse_pop(frm, pc);
@@ -729,6 +735,8 @@ void indent_text(void)
             if ((pc->parent_type == CT_IF) ||
                 (pc->parent_type == CT_ELSE) ||
                 (pc->parent_type == CT_ELSEIF) ||
+                (pc->parent_type == CT_TRY) ||
+                (pc->parent_type == CT_CATCH) ||
                 (pc->parent_type == CT_DO) ||
                 (pc->parent_type == CT_WHILE) ||
                 (pc->parent_type == CT_SWITCH) ||
@@ -858,6 +866,19 @@ void indent_text(void)
 
          /* Always set on case statements */
          indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
+
+         /* comments before 'case' need to be aligned with the 'case' */
+         chunk_t *pct = pc;
+         while (((pct = chunk_get_prev_nnl(pct)) != NULL) &&
+                chunk_is_comment(pct))
+         {
+            chunk_t *t2 = chunk_get_prev(pct);
+            if (chunk_is_newline(t2))
+            {
+               pct->column        = frm.pse[frm.pse_tos].indent_tmp;
+               pct->column_indent = pct->column;
+            }
+         }
       }
       else if (pc->type == CT_BREAK)
       {
@@ -956,7 +977,8 @@ void indent_text(void)
 
          if (((pc->type == CT_FPAREN_OPEN) || (pc->type == CT_ANGLE_OPEN)) &&
              ((cpd.settings[UO_indent_func_call_param].b &&
-               (pc->parent_type == CT_FUNC_CALL))
+               ((pc->parent_type == CT_FUNC_CALL) ||
+                (pc->parent_type == CT_FUNC_CALL_USER)))
               ||
               (cpd.settings[UO_indent_func_proto_param].b &&
                ((pc->parent_type == CT_FUNC_PROTO) ||
@@ -1035,15 +1057,24 @@ void indent_text(void)
                (pc->type == CT_USING))
       {
          /**
-          * if there is a newline after the '=', just indent one level,
+          * if there is a newline after the '=' or the line starts with a '=',
+          * just indent one level,
           * otherwise align on the '='.
-          * Never update indent_column.
           */
+         if ((pc->type == CT_ASSIGN) && chunk_is_newline(chunk_get_prev(pc)))
+         {
+            frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent + indent_size;
+            indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
+            LOG_FMT(LINDENT, "%s: %d] assign => %d [%.*s]\n",
+                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            reindent_line(pc, frm.pse[frm.pse_tos].indent_tmp);
+         }
+
          next = chunk_get_next(pc);
          if (next != NULL)
          {
             indent_pse_push(frm, pc);
-            if (chunk_is_newline(next))
+            if (chunk_is_newline(next) || !cpd.settings[UO_indent_align_assign].b)
             {
                frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
                if (pc->type == CT_ASSIGN)
@@ -1075,7 +1106,8 @@ void indent_text(void)
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos - 1].indent;
          }
       }
-      else if (chunk_is_str(pc, "<<", 2))
+      else if (chunk_is_str(pc, "<<", 2) &&
+               cpd.settings[UO_align_left_shift].b)
       {
          if (cout_col == 0)
          {
@@ -1085,22 +1117,43 @@ void indent_text(void)
       }
       else if (pc->type == CT_THROW)
       {
+         // Pick up what was just before this.
+         prev = chunk_get_prev(pc);
          if (pc->parent_type == CT_FUNC_PROTO)
          {
             indent_pse_push(frm, pc);
-            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
+
+            if ((cpd.settings[UO_indent_func_throw].n != 0) &&
+                ((prev == NULL) || (prev->type == CT_NEWLINE)))
+            {
+               frm.pse[frm.pse_tos].indent = cpd.settings[UO_indent_func_throw].n;
+            }
+            else
+            {
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
+            }
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
             frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
 
             indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
          }
+         else if (pc->parent_type == CT_FUNC_DEF)
+         {
+            if ((cpd.settings[UO_indent_func_throw].n != 0) &&
+                ((prev == NULL) || (prev->type == CT_NEWLINE)))
+            {
+               indent_column_set(cpd.settings[UO_indent_func_throw].n);
+               LOG_FMT(LINDENT, "%s: %d] throw => %d [%.*s]\n",
+                       __func__, pc->orig_line, indent_column, pc->len, pc->str);
+               reindent_line(pc, indent_column);
+            }
+         }
       }
       else
       {
          /* anything else? */
       }
 
-
       /**
        * Indent the line if needed
        */
@@ -1111,7 +1164,6 @@ void indent_text(void)
          LOG_FMT(LINDENT2, "%s: %d] %d for %.*s\n",
                  __func__, pc->orig_line, pc->column_indent, pc->len, pc->str);
 
-
          /**
           * Check for special continuations.
           * Note that some of these could be done as a stack item like
@@ -1119,6 +1171,7 @@ void indent_text(void)
           */
 
          prev = chunk_get_prev_ncnl(pc);
+         next = chunk_get_next_ncnl(pc);
          if ((pc->flags & PCF_DONT_INDENT) != 0)
          {
             /* no change */
@@ -1130,11 +1183,12 @@ void indent_text(void)
             LOG_FMT(LINDENT, "Indent SQL: [%.*s] to %d (%d/%d)\n",
                     pc->len, pc->str, pc->column, sql_col, sql_orig_col);
          }
-         else if ((pc->type == CT_MEMBER) ||
-                  (pc->type == CT_DC_MEMBER) ||
-                  ((prev != NULL) &&
-                   ((prev->type == CT_MEMBER) ||
-                    (prev->type == CT_DC_MEMBER))))
+         else if (((pc->flags & PCF_STMT_START) == 0) &&
+                  ((pc->type == CT_MEMBER) ||
+                   (pc->type == CT_DC_MEMBER) ||
+                   ((prev != NULL) &&
+                    ((prev->type == CT_MEMBER) ||
+                     (prev->type == CT_DC_MEMBER)))))
          {
             tmp = cpd.settings[UO_indent_member].n + indent_column;
             LOG_FMT(LINDENT, "%s: %d] member => %d\n",
@@ -1220,6 +1274,23 @@ void indent_text(void)
                     __func__, pc->orig_line, indent_column, pc->len, pc->str);
             reindent_line(pc, indent_column);
          }
+         else if (cpd.settings[UO_indent_func_const].n &&
+                  (pc->type == CT_QUALIFIER) &&
+                  (strncasecmp(pc->str, "const", pc->len) == 0) &&
+                  ((next == NULL) ||
+                   (next->type == CT_BRACED) ||
+                   (next->type == CT_BRACE_OPEN) ||
+                   (next->type == CT_NEWLINE) ||
+                   (next->type == CT_SEMICOLON) ||
+                   (next->type == CT_THROW) ||
+                   (next->type == CT_VBRACE_OPEN)))
+         {
+            // indent const - void GetFoo(void)\n const\n { return (m_Foo); }
+            indent_column_set(cpd.settings[UO_indent_func_const].n);
+            LOG_FMT(LINDENT, "%s: %d] const => %d [%.*s]\n",
+                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            reindent_line(pc, indent_column);
+         }
          else if (pc->type == CT_BOOL)
          {
             if (cpd.settings[UO_indent_bool_paren].b &&
@@ -1249,6 +1320,34 @@ void indent_text(void)
             sql_col      = pc->column;
             sql_orig_col = pc->orig_col;
          }
+
+         /* Handle indent for variable defs at the top of a block of code */
+         if (pc->flags & PCF_VAR_TYPE)
+         {
+            if (!frm.pse[frm.pse_tos].non_vardef &&
+                (frm.pse[frm.pse_tos].type == CT_BRACE_OPEN))
+            {
+               int tmp = indent_column;
+               if (cpd.settings[UO_indent_var_def_blk].n > 0)
+               {
+                  tmp = cpd.settings[UO_indent_var_def_blk].n;
+               }
+               else
+               {
+                  tmp += cpd.settings[UO_indent_var_def_blk].n;
+               }
+               reindent_line(pc, tmp);
+               LOG_FMT(LINDENT, "%s: %d] var_type indent => %d [%.*s]\n",
+                       __func__, pc->orig_line, tmp, pc->len, pc->str);
+            }
+         }
+         else
+         {
+            if (pc != frm.pse[frm.pse_tos].pc)
+            {
+               frm.pse[frm.pse_tos].non_vardef = true;
+            }
+         }
       }
 
       /**
@@ -1417,7 +1516,8 @@ static void indent_comment(chunk_t *pc, int col)
            pc->orig_line, pc->orig_col, pc->level);
 
    /* force column 1 comment to column 1 if not changing them */
-   if ((pc->orig_col == 1) && !cpd.settings[UO_indent_col1_comment].b)
+   if ((pc->orig_col == 1) && !cpd.settings[UO_indent_col1_comment].b &&
+       ((pc->flags & PCF_INSERTED) == 0))
    {
       LOG_FMT(LCMTIND, "rule 1 - keep in col 1\n");
       reindent_line(pc, 1);
