@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: log_levels.h 1300 2008-07-18 22:17:50Z bengardner $
+ * $Id: log_levels.h 1419 2008-11-14 00:41:29Z bengardner $
  */
 #ifndef LOG_LEVELS_H_INCLUDED
 #define LOG_LEVELS_H_INCLUDED
@@ -83,6 +83,9 @@ enum
    LALAGAIN  = 67,    /* align again */
    LOPERATOR = 68,    /* operator */
    LASFCP    = 69,    /* Align Same Function Call Params */
+   LINDLINED = 70,    /* indent line details */
+   LBCTRL    = 71,    /* beautifier control */
+   LRMRETURN = 72,    /* remove 'return;' */
 };
 
 #endif   /* LOG_LEVELS_H_INCLUDED */
