@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: newlines.cpp 1294 2008-07-18 01:04:49Z bengardner $
+ * $Id: newlines.cpp 1493 2009-02-27 03:45:07Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -21,11 +21,28 @@ static void newlines_double_space_struct_enum_union(chunk_t *open_brace);
 static bool one_liner_nl_ok(chunk_t *pc);
 static void nl_handle_define(chunk_t *pc);
 
+/**
+ * Check to see if we are allowed to increase the newline count.
+ * The only case where we can't is if nl_squeeze_ifdef=true and a preproc is
+ * after the newline.
+ */
+static bool can_increase_nl(chunk_t *nl)
+{
+   if (cpd.settings[UO_nl_squeeze_ifdef].b)
+   {
+      chunk_t *pc = chunk_get_next(nl);
+
+      if ((pc != NULL) && (pc->type == CT_PREPROC) &&
+          (pc->parent_type == CT_PP_ENDIF))
+      {
+         return(false);
+      }
+   }
+   return(true);
+}
 
 /**
- * Check to see if we are allowed to double the newline.
- * The only case where we can't is if nl_squeeze_ifdef=true and a preproc is
- * after the next newline.
+ * Double the newline, if allowed.
  */
 static void double_newline(chunk_t *nl)
 {
@@ -34,18 +51,10 @@ static void double_newline(chunk_t *nl)
    LOG_FMT(LNEWLINE, "%s: add newline after %.*s on line %d",
            __func__, prev->len, prev->str, prev->orig_line);
 
-   if (cpd.settings[UO_nl_squeeze_ifdef].b)
+   if (!can_increase_nl(nl))
    {
-      chunk_t *pc = chunk_get_next(nl);
-
-      LOG_FMT(LNEWLINE, " [%s]", get_token_name(pc->type));
-
-      if ((pc != NULL) && (pc->type == CT_PREPROC) &&
-          (pc->parent_type == CT_PP_ENDIF))
-      {
-         LOG_FMT(LNEWLINE, " - denied\n");
-         return;
-      }
+      LOG_FMT(LNEWLINE, " - denied\n");
+      return;
    }
    LOG_FMT(LNEWLINE, " - done\n");
    nl->nl_count = 2;
@@ -163,7 +172,7 @@ static void newline_min_after2(chunk_t *ref, INT32 count,
    }
    else
    {
-      if (chunk_is_newline(pc))
+      if (chunk_is_newline(pc) && can_increase_nl(pc))
       {
          if (pc->nl_count < count)
          {
@@ -337,7 +346,6 @@ static void newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
       {
          if (cpd.settings[UO_nl_multi_line_cond].b)
          {
-            nl_opt = AV_REMOVE;
             while ((pc = chunk_get_next(pc)) != close_paren)
             {
                if (chunk_is_newline(pc))
@@ -348,14 +356,14 @@ static void newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
             }
          }
 
+         if (nl_opt & AV_REMOVE)
+         {
+            newline_del_between(close_paren, brace_open);
+         }
          if (nl_opt & AV_ADD)
          {
             newline_add_between(close_paren, brace_open);
          }
-         else if (nl_opt & AV_REMOVE)
-         {
-            newline_del_between(close_paren, brace_open);
-         }
 
          newline_add_between(brace_open, chunk_get_next_ncnl(brace_open));
 
@@ -666,7 +674,7 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
  *
  * "struct [name] {" or "struct [name] \n {"
  */
-static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt)
+static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt, bool leave_trailing)
 {
    chunk_t *pc;
    chunk_t *next;
@@ -702,19 +710,21 @@ static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt)
       {
          next = chunk_get_next(next);
       }
-      if (!chunk_is_comment(next) && !chunk_is_newline(next))
+      if (leave_trailing &&
+          !chunk_is_comment(next) &&
+          !chunk_is_newline(next))
       {
          nl_opt = AV_IGNORE;
       }
 
+      if ((nl_opt & AV_REMOVE) != 0)
+      {
+         newline_del_between(start, pc);
+      }
       if ((nl_opt & AV_ADD) != 0)
       {
          newline_add_between(start, pc);
       }
-      else if ((nl_opt & AV_REMOVE) != 0)
-      {
-         newline_del_between(start, pc);
-      }
    }
 }
 
@@ -931,7 +941,9 @@ static void newlines_brace_pair(chunk_t *br_open)
    /* Handle the cases where the brace is part of a function call or definition */
    if ((br_open->parent_type == CT_FUNC_DEF) ||
        (br_open->parent_type == CT_FUNC_CALL) ||
-       (br_open->parent_type == CT_FUNC_CLASS))
+       (br_open->parent_type == CT_FUNC_CALL_USER) ||
+       (br_open->parent_type == CT_FUNC_CLASS) ||
+       (br_open->parent_type == CT_OC_MSG_DECL))
    {
       /* Need to force a newline before the close brace, if not in a class body */
       if ((br_open->flags & PCF_IN_CLASS) == 0)
@@ -944,7 +956,8 @@ static void newlines_brace_pair(chunk_t *br_open)
       newline_add_between(br_open, pc);
 
       val = ((br_open->parent_type == CT_FUNC_DEF) ||
-             (br_open->parent_type == CT_FUNC_CLASS)) ?
+             (br_open->parent_type == CT_FUNC_CLASS) ||
+             (br_open->parent_type == CT_OC_MSG_DECL)) ?
             cpd.settings[UO_nl_fdef_brace].a :
             cpd.settings[UO_nl_fcall_brace].a;
 
@@ -1157,27 +1170,47 @@ static void newline_func_def(chunk_t *start)
    }
 
    /* Handle break newlines type and function */
-   argval_t a = (start->parent_type == CT_FUNC_PROTO) ?
-                cpd.settings[UO_nl_func_proto_type_name].a :
-                cpd.settings[UO_nl_func_type_name].a;
+   prev = chunk_get_prev_ncnl(start);
+   prev = chunk_get_prev_ncnl(prev);
 
-   if (a != AV_IGNORE)
+   if (prev != NULL)
    {
-      prev = chunk_get_prev_ncnl(start);
-      prev = chunk_get_prev_ncnl(prev);
-
-      /* If we are on a '::', step back two tokens
-       * TODO: do we also need to check for '.' ?
-       */
-      while ((prev != NULL) && (prev->type == CT_DC_MEMBER))
+      if (prev->type == CT_OPERATOR)
       {
-         prev = chunk_get_prev_ncnl(prev);
+         tmp  = prev;
          prev = chunk_get_prev_ncnl(prev);
       }
-
-      if (prev != NULL)
+      else
       {
-         newline_iarf(prev, a);
+         tmp = start;
+      }
+      if ((prev != NULL) && (prev->type == CT_DC_MEMBER))
+      {
+         if (cpd.settings[UO_nl_func_scope_name].a != AV_IGNORE)
+         {
+            newline_iarf(prev, cpd.settings[UO_nl_func_scope_name].a);
+         }
+      }
+
+      argval_t a = (tmp->parent_type == CT_FUNC_PROTO) ?
+          cpd.settings[UO_nl_func_proto_type_name].a :
+          cpd.settings[UO_nl_func_type_name].a;
+
+      if (a != AV_IGNORE)
+      {
+         /* If we are on a '::', step back two tokens
+          * TODO: do we also need to check for '.' ?
+          */
+         while ((prev != NULL) && (prev->type == CT_DC_MEMBER))
+         {
+            prev = chunk_get_prev_ncnl(prev);
+            prev = chunk_get_prev_ncnl(prev);
+         }
+
+         if (prev != NULL)
+         {
+            newline_iarf(prev, a);
+         }
       }
    }
 
@@ -1404,7 +1437,11 @@ void newlines_cleanup_braces(void)
          }
 
          next = chunk_get_next_nnl(pc);
-         if (next->type == CT_BRACE_CLOSE)
+         if (next == NULL)
+         {
+            // do nothing
+         }
+         else if (next->type == CT_BRACE_CLOSE)
          {
             //TODO: add an option to split open empty statements? { };
          }
@@ -1484,11 +1521,15 @@ void newlines_cleanup_braces(void)
          }
 
          /* Force a newline after a function def */
-         if (pc->parent_type == CT_FUNC_DEF)
+         if (cpd.settings[UO_nl_after_brace_close].b ||
+             (pc->parent_type == CT_FUNC_DEF) ||
+             (pc->parent_type == CT_OC_MSG_DECL))
          {
             next = chunk_get_next(pc);
             if ((next != NULL) &&
                 (next->type != CT_SEMICOLON) &&
+                (next->type != CT_COMMA) &&
+                ((pc->flags & (PCF_IN_ARRAY_ASSIGN | PCF_IN_TYPEDEF)) == 0) &&
                 !chunk_is_newline(next) &&
                 !chunk_is_comment(next))
             {
@@ -1521,15 +1562,15 @@ void newlines_cleanup_braces(void)
       }
       else if (pc->type == CT_STRUCT)
       {
-         newlines_struct_enum_union(pc, cpd.settings[UO_nl_struct_brace].a);
+         newlines_struct_enum_union(pc, cpd.settings[UO_nl_struct_brace].a, true);
       }
       else if (pc->type == CT_UNION)
       {
-         newlines_struct_enum_union(pc, cpd.settings[UO_nl_union_brace].a);
+         newlines_struct_enum_union(pc, cpd.settings[UO_nl_union_brace].a, true);
       }
       else if (pc->type == CT_ENUM)
       {
-         newlines_struct_enum_union(pc, cpd.settings[UO_nl_enum_brace].a);
+         newlines_struct_enum_union(pc, cpd.settings[UO_nl_enum_brace].a, true);
       }
       else if (pc->type == CT_CASE)
       {
@@ -1593,12 +1634,14 @@ void newlines_cleanup_braces(void)
       else if (pc->type == CT_FPAREN_OPEN)
       {
          if (((pc->parent_type == CT_FUNC_DEF) ||
-              (pc->parent_type == CT_FUNC_PROTO))
+              (pc->parent_type == CT_FUNC_PROTO) ||
+              (pc->parent_type == CT_OPERATOR))
              &&
              ((cpd.settings[UO_nl_func_decl_start].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_decl_args].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_decl_end].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_type_name].a != AV_IGNORE) ||
+              (cpd.settings[UO_nl_func_scope_name].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_proto_type_name].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_paren].a != AV_IGNORE)))
          {
@@ -1623,7 +1666,7 @@ void newlines_cleanup_braces(void)
       }
       else if (pc->type == CT_NAMESPACE)
       {
-         newlines_struct_enum_union(pc, cpd.settings[UO_nl_namespace_brace].a);
+         newlines_struct_enum_union(pc, cpd.settings[UO_nl_namespace_brace].a, false);
       }
       else if (pc->type == CT_SQUARE_OPEN)
       {
@@ -2188,6 +2231,7 @@ void do_blank_lines(void)
       if ((prev != NULL) && (prev->type == CT_BRACE_CLOSE) &&
           ((prev->parent_type == CT_FUNC_DEF) ||
            (prev->parent_type == CT_FUNC_CLASS) ||
+           (prev->parent_type == CT_OC_MSG_DECL) ||
            (prev->parent_type == CT_ASSIGN)))
       {
          if (prev->flags & PCF_ONE_LINER)
@@ -2218,11 +2262,74 @@ void do_blank_lines(void)
          }
          if ((cpd.settings[UO_nl_after_func_proto_group].n > pc->nl_count) &&
              (next != NULL) &&
-             (next->parent_type != CT_FUNC_PROTO))
+             (next->parent_type != CT_FUNC_PROTO) &&
+             can_increase_nl(pc))
          {
             pc->nl_count = cpd.settings[UO_nl_after_func_proto_group].n;
          }
       }
+
+      /* Change blanks between a function comment and body */
+      if ((cpd.settings[UO_nl_comment_func_def].n != 0) &&
+          chunk_is_comment(pcmt) &&
+          (next != NULL) && (next->parent_type == CT_FUNC_DEF))
+      {
+         if (cpd.settings[UO_nl_comment_func_def].n != pc->nl_count)
+         {
+            LOG_FMT(LCMTNL, "%s: nl_comment_func_def affected line %d\n", __func__, pc->orig_line);
+            pc->nl_count = cpd.settings[UO_nl_comment_func_def].n;
+         }
+      }
+
+      /* Change blanks after a try-catch-finally block */
+      if ((cpd.settings[UO_nl_after_try_catch_finally].n != 0) &&
+          (cpd.settings[UO_nl_after_try_catch_finally].n != pc->nl_count) &&
+          (prev != NULL) && (next != NULL))
+      {
+         if ((prev->type == CT_BRACE_CLOSE) &&
+             ((prev->parent_type == CT_CATCH) ||
+              (prev->parent_type == CT_FINALLY)))
+         {
+            if ((next->type != CT_BRACE_CLOSE) &&
+                (next->type != CT_CATCH) &&
+                (next->type != CT_FINALLY))
+            {
+               pc->nl_count = cpd.settings[UO_nl_after_try_catch_finally].n;
+            }
+         }
+      }
+
+      /* Change blanks after a try-catch-finally block */
+      if ((cpd.settings[UO_nl_between_get_set].n != 0) &&
+          (cpd.settings[UO_nl_between_get_set].n != pc->nl_count) &&
+          (prev != NULL) && (next != NULL))
+      {
+         if ((prev->parent_type == CT_GETSET) &&
+             (next->type != CT_BRACE_CLOSE) &&
+             ((prev->type == CT_BRACE_CLOSE) ||
+              (prev->type == CT_SEMICOLON)))
+         {
+            pc->nl_count = cpd.settings[UO_nl_between_get_set].n;
+         }
+      }
+
+      /* Change blanks after a try-catch-finally block */
+      if ((cpd.settings[UO_nl_around_cs_property].n != 0) &&
+          (cpd.settings[UO_nl_around_cs_property].n != pc->nl_count) &&
+          (prev != NULL) && (next != NULL))
+      {
+         if ((prev->type == CT_BRACE_CLOSE) &&
+             (prev->parent_type == CT_CS_PROPERTY) &&
+             (next->type != CT_BRACE_CLOSE))
+         {
+            pc->nl_count = cpd.settings[UO_nl_around_cs_property].n;
+         }
+         else if ((next->parent_type == CT_CS_PROPERTY) &&
+                  (next->flags & PCF_STMT_START))
+         {
+            pc->nl_count = cpd.settings[UO_nl_around_cs_property].n;
+         }
+      }
    }
 }
 
