@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: options.cpp 1300 2008-07-18 22:17:50Z bengardner $
+ * $Id: options.cpp 1495 2009-02-28 17:37:10Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "args.h"
@@ -135,8 +135,16 @@ void register_options(void)
                   "Add or remove space around arithmetic operator '+', '-', '/', '*', etc");
    unc_add_option("sp_assign", UO_sp_assign, AT_IARF,
                   "Add or remove space around assignment operator '=', '+=', etc");
+   unc_add_option("sp_before_assign", UO_sp_before_assign, AT_IARF,
+                  "Add or remove space before assignment operator '=', '+=', etc. Overrides sp_assign.");
+   unc_add_option("sp_after_assign", UO_sp_after_assign, AT_IARF,
+                  "Add or remove space after assignment operator '=', '+=', etc. Overrides sp_assign.");
    unc_add_option("sp_enum_assign", UO_sp_enum_assign, AT_IARF,
                   "Add or remove space around assignment '=' in enum");
+   unc_add_option("sp_enum_before_assign", UO_sp_enum_before_assign, AT_IARF,
+                  "Add or remove space before assignment '=' in enum. Overrides sp_enum_assign.");
+   unc_add_option("sp_enum_after_assign", UO_sp_enum_after_assign, AT_IARF,
+                  "Add or remove space after assignment '=' in enum. Overrides sp_enum_assign.");
    unc_add_option("sp_bool", UO_sp_bool, AT_IARF,
                   "Add or remove space around boolean operators '&&' and '||'");
    unc_add_option("sp_compare", UO_sp_compare, AT_IARF,
@@ -158,13 +166,21 @@ void register_options(void)
                   "Add or remove space between pointer stars '*'");
    unc_add_option("sp_after_ptr_star", UO_sp_after_ptr_star, AT_IARF,
                   "Add or remove space after pointer star '*', if followed by a word.");
+   unc_add_option("sp_after_ptr_star_func", UO_sp_after_ptr_star_func, AT_IARF,
+                  "Add or remove space after a pointer star '*', if followed by a func proto/def.");
+   unc_add_option("sp_before_ptr_star_func", UO_sp_before_ptr_star_func, AT_IARF,
+                  "Add or remove space before a pointer star '*', if followed by a func proto/def.");
    unc_add_option("sp_before_byref", UO_sp_before_byref, AT_IARF,
-                  "Add or remove space before reference sign '&'");
+                  "Add or remove space before a reference sign '&'");
    unc_add_option("sp_before_unnamed_byref", UO_sp_before_unnamed_byref, AT_IARF,
-                  "Add or remove space before reference sign '&' that isn't followed by a variable name\n"
+                  "Add or remove space before a reference sign '&' that isn't followed by a variable name\n"
                   "If set to 'ignore', sp_before_byref is used instead.");
    unc_add_option("sp_after_byref", UO_sp_after_byref, AT_IARF,
                   "Add or remove space after reference sign '&', if followed by a word.");
+   unc_add_option("sp_after_byref_func", UO_sp_after_byref_func, AT_IARF,
+                  "Add or remove space after a reference sign '&', if followed by a func proto/def.");
+   unc_add_option("sp_before_byref_func", UO_sp_before_byref_func, AT_IARF,
+                  "Add or remove space before a reference sign '&', if followed by a func proto/def.");
    unc_add_option("sp_after_type", UO_sp_after_type, AT_IARF,
                   "Add or remove space between type and word");
    unc_add_option("sp_template_angle", UO_sp_template_angle, AT_IARF,
@@ -184,10 +200,16 @@ void register_options(void)
                   "Add or remove space before '(' of 'if', 'for', 'switch', and 'while'");
    unc_add_option("sp_inside_sparen", UO_sp_inside_sparen, AT_IARF,
                   "Add or remove space inside if-condition '(' and ')'");
+   unc_add_option("sp_inside_sparen_close", UO_sp_inside_sparen_close, AT_IARF,
+                  "Add or remove space before if-condition ')'. Overrides sp_inside_sparen.");
    unc_add_option("sp_after_sparen", UO_sp_after_sparen, AT_IARF,
                   "Add or remove space after ')' of 'if', 'for', 'switch', and 'while'");
    unc_add_option("sp_sparen_brace", UO_sp_sparen_brace, AT_IARF,
                   "Add or remove space between ')' and '{' of 'if', 'for', 'switch', and 'while'");
+   unc_add_option("sp_invariant_paren", UO_sp_invariant_paren, AT_IARF,
+                  "Add or remove space between 'invariant' and '(' in the D language.");
+   unc_add_option("sp_after_invariant_paren", UO_sp_after_invariant_paren, AT_IARF,
+                  "Add or remove space after the ')' in 'invariant (C) c' in the D language.");
    unc_add_option("sp_special_semi", UO_sp_special_semi, AT_IARF,
                   "Add or remove space before empty statement ';' on 'if', 'for' and 'while'");
    unc_add_option("sp_before_semi", UO_sp_before_semi, AT_IARF,
@@ -212,6 +234,8 @@ void register_options(void)
                   "Add or remove space after class ':'");
    unc_add_option("sp_before_class_colon", UO_sp_before_class_colon, AT_IARF,
                   "Add or remove space before class ':'");
+   unc_add_option("sp_before_case_colon", UO_sp_before_case_colon, AT_IARF,
+                  "Add or remove space before case ':'");
    unc_add_option("sp_after_operator", UO_sp_after_operator, AT_IARF,
                   "Add or remove space between 'operator' and operator sign");
    unc_add_option("sp_after_operator_sym", UO_sp_after_operator_sym, AT_IARF,
@@ -220,6 +244,8 @@ void register_options(void)
                   "Add or remove space after C/D cast, ie 'cast(int)a' vs 'cast(int) a' or '(int)a' vs '(int) a'");
    unc_add_option("sp_inside_paren_cast", UO_sp_inside_paren_cast, AT_IARF,
                   "Add or remove spaces inside cast parens");
+   unc_add_option("sp_cpp_cast_paren", UO_sp_cpp_cast_paren, AT_IARF,
+                  "Add or remove space between the type and open paren in a C++ cast, ie 'int(exp)' vs 'int (exp)'");
    unc_add_option("sp_sizeof_paren", UO_sp_sizeof_paren, AT_IARF,
                   "Add or remove space between 'sizeof' and '('");
    unc_add_option("sp_after_tag", UO_sp_after_tag, AT_IARF,
@@ -249,6 +275,9 @@ void register_options(void)
                   "Add or remove space between ')' and '{' of function");
    unc_add_option("sp_func_call_paren", UO_sp_func_call_paren, AT_IARF,
                   "Add or remove space between function name and '(' on function calls");
+   unc_add_option("sp_func_call_user_paren", UO_sp_func_call_user_paren, AT_IARF,
+                  "Add or remove space between the user function name and '(' on function calls\n"
+                  "You need to set a keyword to be a user function, like this: 'set func_call_user _' in the config file.");
    unc_add_option("sp_func_class_paren", UO_sp_func_class_paren, AT_IARF,
                   "Add or remove space between a constructor/destructor and the open paren");
    unc_add_option("sp_return_paren", UO_sp_return_paren, AT_IARF,
@@ -257,6 +286,8 @@ void register_options(void)
                   "Add or remove space between '__attribute__' and '('");
    unc_add_option("sp_defined_paren", UO_sp_defined_paren, AT_IARF,
                   "Add or remove space between 'defined' and '(' in '#if defined (FOO)'");
+   unc_add_option("sp_throw_paren", UO_sp_throw_paren, AT_IARF,
+                  "Add or remove space between 'throw' and '(' in 'throw (something)'");
    unc_add_option("sp_macro", UO_sp_macro, AT_IARF,
                   "Add or remove space between macro and value");
    unc_add_option("sp_macro_func", UO_sp_macro_func, AT_IARF,
@@ -265,6 +296,8 @@ void register_options(void)
                   "Add or remove space between 'else' and '{' if on the same line");
    unc_add_option("sp_brace_else", UO_sp_brace_else, AT_IARF,
                   "Add or remove space between '}' and 'else' if on the same line");
+   unc_add_option("sp_brace_typedef", UO_sp_brace_typedef, AT_IARF,
+                  "Add or remove space between '}' and the name of a typedef on the same line");
    unc_add_option("sp_catch_brace", UO_sp_catch_brace, AT_IARF,
                   "Add or remove space between 'catch' and '{' if on the same line");
    unc_add_option("sp_brace_catch", UO_sp_brace_catch, AT_IARF,
@@ -304,18 +337,33 @@ void register_options(void)
 
    unc_add_option("sp_after_oc_scope", UO_sp_after_oc_scope, AT_IARF,
                   "Add or remove space after the scope '+' or '-', as in '-(void) foo;' or '+(int) bar;'");
-   unc_add_option("sp_before_oc_colon", UO_sp_before_oc_colon, AT_IARF,
+   unc_add_option("sp_after_oc_colon", UO_sp_after_oc_colon, AT_IARF,
                   "Add or remove space after the colon in message specs\n"
-                  "'-(int) f: (int) x;' vs '+(int) f : (int) x;'");
+                  "'-(int) f:(int) x;' vs '-(int) f: (int) x;'");
+   unc_add_option("sp_before_oc_colon", UO_sp_before_oc_colon, AT_IARF,
+                  "Add or remove space before the colon in message specs\n"
+                  "'-(int) f: (int) x;' vs '-(int) f : (int) x;'");
+   unc_add_option("sp_after_send_oc_colon", UO_sp_after_send_oc_colon, AT_IARF,
+                  "Add or remove space after the colon in message specs\n"
+                  "'[object setValue:1];' vs '[object setValue: 1];'");
+   unc_add_option("sp_before_send_oc_colon", UO_sp_before_send_oc_colon, AT_IARF,
+                  "Add or remove space before the colon in message specs\n"
+                  "'[object setValue:1];' vs '[object setValue :1];'");
    unc_add_option("sp_after_oc_type", UO_sp_after_oc_type, AT_IARF,
                   "Add or remove space after the (type) in message specs\n"
-                  "'-(int) f: (int) x;' vs '+(int)f : (int)x;'");
+                  "'-(int) f: (int) x;' vs '-(int) f: (int)x;'");
 
    unc_add_option("sp_cond_colon", UO_sp_cond_colon, AT_IARF,
                   "Add or remove space around the ':' in 'b ? t : f'");
    unc_add_option("sp_cond_question", UO_sp_cond_question, AT_IARF,
                   "Add or remove space around the '?' in 'b ? t : f'");
+   unc_add_option("sp_case_label", UO_sp_case_label, AT_IARF,
+                  "Fix the spacing between 'case' and the label. Only 'ignore' and 'force' make sense here.");
+   unc_add_option("sp_range", UO_sp_range, AT_IARF,
+                  "Control the space around the D '..' operator.");
 
+   unc_add_option("sp_cmt_cpp_start", UO_sp_cmt_cpp_start, AT_IARF,
+                  "Control the space after the opening of a C++ comment '// A' vs '//A'");
 
    unc_begin_group(UG_indent, "Indenting");
    unc_add_option("indent_columns", UO_indent_columns, AT_NUM,
@@ -350,6 +398,8 @@ void register_options(void)
    unc_add_option("indent_else_if", UO_indent_else_if, AT_BOOL,
                   "False=treat 'else\\nif' as 'else if' for indenting purposes\n"
                   "True=indent the 'if' one level\n");
+   unc_add_option("indent_var_def_blk", UO_indent_var_def_blk, AT_NUM,
+                  "Amount to indent variable declarations after a open brace. neg=relative, pos=absolute");
 
    unc_add_option("indent_func_call_param", UO_indent_func_call_param, AT_BOOL,
                   "True:  indent continued function call parameters one indent level\n"
@@ -367,6 +417,11 @@ void register_options(void)
    unc_add_option("indent_func_param_double", UO_indent_func_param_double, AT_BOOL,
                   "Double the indent for indent_func_xxx_param options");
 
+   unc_add_option("indent_func_const", UO_indent_func_const, AT_NUM,
+                  "Indentation column for standalone 'const' function decl/proto qualifier");
+   unc_add_option("indent_func_throw", UO_indent_func_throw, AT_NUM,
+                  "Indentation column for standalone 'throw' function decl/proto qualifier");
+
    unc_add_option("indent_member", UO_indent_member, AT_NUM,
                   "The number of spaces to indent a continued '->' or '.'\n"
                   "Usually set to 0, 1, or indent_columns.");
@@ -413,6 +468,9 @@ void register_options(void)
                   "If an open square is followed by a newline, indent the next line so that it lines up after the open square (not recommended)");
    unc_add_option("indent_preserve_sql", UO_indent_preserve_sql, AT_BOOL,
                   "Don't change the relative indent of ESQL/C 'EXEC SQL' bodies");
+   unc_add_option("indent_align_assign", UO_indent_align_assign, AT_BOOL,
+                  "Align continued statements at the '='. Default=True\n"
+                  "If FALSE or the '=' is followed by a newline, the next line is indent one tab.");
 
    unc_begin_group(UG_newline, "Newline adding and removing options");
    unc_add_option("nl_collapse_empty_body", UO_nl_collapse_empty_body, AT_BOOL,
@@ -490,7 +548,8 @@ void register_options(void)
    unc_add_option("nl_switch_brace", UO_nl_switch_brace, AT_IARF,
                   "Add or remove newline between 'switch' and '{'");
    unc_add_option("nl_multi_line_cond", UO_nl_multi_line_cond, AT_BOOL,
-                  "Add or remove newline when condition spans two or more lines");
+                  "Add a newline between ')' and '{' if the ')' is on a different line than the if/for/etc.\n"
+                  "Overrides nl_for_brace, nl_if_brace, nl_switch_brace, nl_while_switch, and nl_catch_brace.");
    unc_add_option("nl_multi_line_define", UO_nl_multi_line_define, AT_BOOL,
                   "Force a newline in a define after the macro name for multi-line defines.");
    unc_add_option("nl_before_case", UO_nl_before_case, AT_BOOL,
@@ -509,6 +568,9 @@ void register_options(void)
                   "Add or remove newline after each ',' in the constructor member initialization");
    unc_add_option("nl_func_type_name", UO_nl_func_type_name, AT_IARF,
                   "Add or remove newline between return type and function name in definition");
+   unc_add_option("nl_func_scope_name", UO_nl_func_scope_name, AT_IARF,
+                  "Add or remove newline between function scope and name in a definition\n"
+                  "Controls the newline after '::' in 'void A::f() { }'");
    unc_add_option("nl_func_proto_type_name", UO_nl_func_proto_type_name, AT_IARF,
                   "Add or remove newline between return type and function name in a prototype");
    unc_add_option("nl_func_paren", UO_nl_func_paren, AT_IARF,
@@ -536,6 +598,9 @@ void register_options(void)
    unc_add_option("nl_after_vbrace_open", UO_nl_after_vbrace_open, AT_BOOL,
                   "Whether to put a newline after a virtual brace open.\n"
                   "These occur in un-braced if/while/do/for statement bodies.");
+   unc_add_option("nl_after_brace_close", UO_nl_after_brace_close, AT_BOOL,
+                  "Whether to put a newline after a brace close.\n"
+                  "Does not apply if followed by a necessary ';'.");
    unc_add_option("nl_define_macro", UO_nl_define_macro, AT_BOOL,
                   "Whether to alter newlines in '#define' macros");
    unc_add_option("nl_squeeze_ifdef", UO_nl_squeeze_ifdef, AT_BOOL,
@@ -608,12 +673,30 @@ void register_options(void)
                   "The number of newlines after a 'private:', 'public:', 'protected:', 'signals:', or 'slots:' label.\n"
                   "0 = No change.");
 
+   unc_add_option("nl_comment_func_def", UO_nl_comment_func_def, AT_NUM,
+                  "The number of newlines between a function def and the function comment.\n"
+                  "0 = No change.");
+
+   unc_add_option("nl_after_try_catch_finally", UO_nl_after_try_catch_finally, AT_NUM,
+                  "The number of newlines after a try-catch-finally block that isn't followed by a brace close.\n"
+                  "0 = No change.");
+   unc_add_option("nl_around_cs_property", UO_nl_around_cs_property, AT_NUM,
+                  "The number of newlines before and after a property, indexer or event decl.\n"
+                  "0 = No change.");
+   unc_add_option("nl_between_get_set", UO_nl_between_get_set, AT_NUM,
+                  "The number of newlines between the get/set/add/remove handlers in C#.\n"
+                  "0 = No change.");
+
    unc_add_option("eat_blanks_after_open_brace", UO_eat_blanks_after_open_brace, AT_BOOL,
                   "Whether to remove blank lines after '{'");
    unc_add_option("eat_blanks_before_close_brace", UO_eat_blanks_before_close_brace, AT_BOOL,
                   "Whether to remove blank lines before '}'");
 
    unc_begin_group(UG_position, "Positioning options");
+   unc_add_option("pos_arith", UO_pos_arith, AT_POS,
+                  "The position of arithmetic operators in wrapped expressions");
+   unc_add_option("pos_assign", UO_pos_assign, AT_POS,
+                  "The position of assignment in wrapped expressions");
    unc_add_option("pos_bool", UO_pos_bool, AT_POS,
                   "The position of boolean operators in wrapped expressions");
    unc_add_option("pos_comma", UO_pos_comma, AT_POS,
@@ -649,9 +732,9 @@ void register_options(void)
                   "The span for aligning variable definitions (0=don't align)", "", 0, 5000);
    unc_add_option("align_var_def_star_style", UO_align_var_def_star_style, AT_NUM,
                   "How to align the star in variable definitions.\n"
-                  " 0=Part of the type\n"
-                  " 1=Part of the variable\n"
-                  " 2=Dangling", "", 0, 2);
+                  " 0=Part of the type     'void *   foo;'\n"
+                  " 1=Part of the variable 'void     *foo;'\n"
+                  " 2=Dangling             'void    *foo;'", "", 0, 2);
    unc_add_option("align_var_def_amp_style", UO_align_var_def_amp_style, AT_NUM,
                   "How to align the '&' in variable definitions.\n"
                   " 0=Part of the type\n"
@@ -713,6 +796,8 @@ void register_options(void)
                   "The span for aligning function prototypes (0=don't align)", "", 0, 5000);
    unc_add_option("align_func_proto_gap", UO_align_func_proto_gap, AT_NUM,
                   "Minimum gap between the return type and the function name.");
+   unc_add_option("align_on_operator", UO_align_on_operator, AT_BOOL,
+                  "Align function protos on the 'operator' keyword instead of what follows");
    unc_add_option("align_mix_var_proto", UO_align_mix_var_proto, AT_BOOL,
                   "Whether to mix aligning prototype and variable declarations.\n"
                   "If true, align_var_def_XXX options are used instead of align_func_proto_XXX options.");
@@ -732,6 +817,8 @@ void register_options(void)
                   "The minimum space between label and value of a preprocessor define");
    unc_add_option("align_pp_define_span", UO_align_pp_define_span, AT_NUM,
                   "The span for aligning on '#define' bodies (0=don't align)", "", 0, 5000);
+   unc_add_option("align_left_shift", UO_align_left_shift, AT_BOOL,
+                  "Align lines that start with '<<' with previous '<<'. Default=true");
 
    unc_begin_group(UG_comment, "Comment modifications");
    unc_add_option("cmt_width", UO_cmt_width, AT_NUM,
@@ -767,6 +854,9 @@ void register_options(void)
    unc_add_option("cmt_insert_file_header", UO_cmt_insert_file_header, AT_STRING,
                   "The filename that contains text to insert at the head of a file if the file doesn't start with a C/C++ comment.\n"
                   "Will substitue $(filename) with the current file's name.");
+   unc_add_option("cmt_insert_file_footer", UO_cmt_insert_file_footer, AT_STRING,
+                  "The filename that contains text to insert at the end of a file if the file doesn't end with a C/C++ comment.\n"
+                  "Will substitue $(filename) with the current file's name.");
    unc_add_option("cmt_insert_func_header", UO_cmt_insert_func_header, AT_STRING,
                   "The filename that contains text to insert before a function implementation if the function isn't preceeded with a C/C++ comment.\n"
                   "Will substitue $(function) with the function name and $(javaparam) with the javadoc @param and @return stuff.\n"
@@ -774,6 +864,10 @@ void register_options(void)
    unc_add_option("cmt_insert_class_header", UO_cmt_insert_class_header, AT_STRING,
                   "The filename that contains text to insert before a class if the class isn't preceeded with a C/C++ comment.\n"
                   "Will substitue $(class) with the class name.");
+   unc_add_option("cmt_insert_before_preproc", UO_cmt_insert_before_preproc, AT_BOOL,
+                  "If a preprocessor is encountered when stepping backwards from a function name, then\n"
+                  "this option decides whether the comment should be inserted.\n"
+                  "Affects cmt_insert_func_header and cmt_insert_class_header.");
 
    unc_begin_group(UG_codemodify, "Code modifying options (non-whitespace)");
    unc_add_option("mod_full_brace_do", UO_mod_full_brace_do, AT_IARF,
@@ -811,6 +905,8 @@ void register_options(void)
                   "This is generally a bad idea, as it may break your code.");
    unc_add_option("mod_move_case_break", UO_mod_move_case_break, AT_BOOL,
                   "If TRUE, it will move a 'break' that appears after a fully braced 'case' before the close brace.");
+   unc_add_option("mod_remove_empty_return", UO_mod_remove_empty_return, AT_BOOL,
+                  "If TRUE, it will remove a void 'return;' that appears as the last statement in a function.");
 
    unc_begin_group(UG_preprocessor, "Preprocessor options");
    unc_add_option("pp_indent", UO_pp_indent, AT_IARF,
@@ -1125,6 +1221,28 @@ int load_option_file(const char *filename)
       {
          add_keyword(args[1], CT_MACRO_ELSE, LANG_ALL);
       }
+      else if (strcasecmp(args[0], "set") == 0)
+      {
+         if (argc < 3)
+         {
+            LOG_FMT(LWARN, "%s:%d 'set' requires at least three arguments\n",
+                    filename, cpd.line_number);
+         }
+         else
+         {
+            c_token_t id = find_token_name(args[1]);
+            if (id != CT_NONE)
+            {
+               LOG_FMT(LNOTE, "%s:%d set '%s':", filename, cpd.line_number, args[1]);
+               for (idx = 2; idx < argc; idx++)
+               {
+                  LOG_FMT(LNOTE, " '%s'", args[idx]);
+                  add_keyword(args[idx], id, LANG_ALL);
+               }
+               LOG_FMT(LNOTE, "\n");
+            }
+         }
+      }
       else
       {
          /* must be a regular option = value */
@@ -1218,12 +1336,15 @@ int save_option_file(FILE *pfile, bool withDoc)
               "# Example:\n"
               "# type myfoo1 myfoo2\n"
               "#\n"
-              "# You can create custom macro-based indentation using macro-open, \n"
+              "# You can create custom macro-based indentation using macro-open,\n"
               "# macro-else and macro-close.\n"
               "# Example:\n"
               "# macro-open  BEGIN_TEMPLATE_MESSAGE_MAP\n"
               "# macro-open  BEGIN_MESSAGE_MAP\n"
               "# macro-close END_MESSAGE_MAP\n"
+              "#\n"
+              "# You can assign any keyword to any type with the set option.\n"
+              "# set func_call_user _ N_\n"
               );
    }
 
@@ -1254,7 +1375,10 @@ int save_option_file(FILE *pfile, bool withDoc)
       }
       else
       {
-         LOG_FMT(LWARN, "%s: unable to save '%s'\n", __func__, ct->tag);
+         const char *tn = get_token_name(ct->type);
+
+         fprintf(pfile, "set %s %*.s%s\n", tn,
+                 int(cpd.max_option_name_len - (4 + strlen(tn))), " ", ct->tag);
       }
    }
 
@@ -1361,9 +1485,12 @@ void set_option_defaults(void)
    cpd.settings[UO_sp_incdec].a            = AV_REMOVE;
    cpd.settings[UO_sp_after_type].a        = AV_FORCE;
    cpd.settings[UO_sp_before_nl_cont].a    = AV_ADD;
+   cpd.settings[UO_sp_before_case_colon].a = AV_REMOVE;
    cpd.settings[UO_cmt_indent_multi].b     = true;
    cpd.settings[UO_cmt_multi_check_last].b = true;
    cpd.settings[UO_pp_indent_count].n      = 1;
+   cpd.settings[UO_align_left_shift].b     = true;
+   cpd.settings[UO_indent_align_assign].b  = true;
 }
 
 std::string argtype_to_string(argtype_e argtype)
