@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: options.h 1300 2008-07-18 22:17:50Z bengardner $
+ * $Id: options.h 1495 2009-02-28 17:37:10Z bengardner $
  */
 #ifndef OPTIONS_H_INCLUDED
 #define OPTIONS_H_INCLUDED
@@ -138,6 +138,9 @@ enum uncrustify_options
    UO_indent_template_param,
    UO_indent_func_param_double,  // double the tab indent for
 
+   UO_indent_func_const,         // indentation for standalone "const" qualifier
+   UO_indent_func_throw,         // indentation for standalone "throw" qualifier
+
    UO_indent_namespace,          // indent stuff inside namespace braces
    UO_indent_extern,
    UO_indent_class,              // indent stuff inside class braces
@@ -148,8 +151,10 @@ enum uncrustify_options
    UO_indent_sing_line_comments, // indent single line ('//') comments on lines before code
    UO_indent_relative_single_line_comments, // indent single line ('//') comments after code
    UO_indent_preserve_sql,       // preserve indent of EXEC SQL statement body
+   UO_indent_align_assign,
 
    UO_indent_else_if,
+   UO_indent_var_def_blk,        // indent a variable def block that appears at the top
 
    /*
     * Misc inter-element spacing
@@ -161,6 +166,7 @@ enum uncrustify_options
 
    UO_sp_after_cast,            // space after C & D cast - "(int) a" vs "(int)a"
    UO_sp_inside_paren_cast,     // spaces inside the parens of a cast
+   UO_sp_cpp_cast_paren,
 
    UO_sp_before_byref,          // space before '&' of 'fcn(int& idx)'
    UO_sp_before_unnamed_byref,
@@ -173,11 +179,15 @@ enum uncrustify_options
    UO_sp_inside_paren,          // space inside '+ ( xxx )' vs '+ (xxx)'
    UO_sp_inside_square,         // space inside 'byte[ 5 ]' vs 'byte[5]'
    UO_sp_inside_sparen,         // space inside 'if( xxx )' vs 'if(xxx)'
+   UO_sp_inside_sparen_close,
    UO_sp_inside_angle,          // space inside '<>', as in '<class T>'
 
    UO_sp_before_sparen,         // space before '(' of 'if/for/while/switch'
-   UO_sp_after_sparen,      /* space after  ')' of 'if/for/while/switch'
-                             * the do-while does not get set here */
+   UO_sp_after_sparen,          /* space after  ')' of 'if/for/while/switch'
+                                 * the do-while does not get set here */
+   UO_sp_after_invariant_paren,
+   UO_sp_invariant_paren,
+
    UO_sp_template_angle,
    UO_sp_before_angle,          // space before '<>', as in '<class T>'
    UO_sp_after_angle,           // space after  '<>', as in '<class T>'
@@ -200,24 +210,34 @@ enum uncrustify_options
    UO_sp_bool,                  // space around || &&
    UO_sp_compare,               // space around < > ==, etc
    UO_sp_assign,                // space around =, +=, etc
+   UO_sp_before_assign,         // space before =, +=, etc
+   UO_sp_after_assign,          // space after =, +=, etc
    UO_sp_enum_assign,           // space around = in enum
-
+   UO_sp_enum_before_assign,    // space before = in enum
+   UO_sp_enum_after_assign,     // space after = in enum
    UO_sp_after_class_colon,     // space after class ':'
    UO_sp_before_class_colon,    // space before class ':'
+   UO_sp_before_case_colon,
 
    UO_sp_func_def_paren,        // space between 'func' and '(' - "foo (" vs "foo("
    UO_sp_func_call_paren,       // space between 'func' and '(' - "foo (" vs "foo("
+   UO_sp_func_call_user_paren,
    UO_sp_func_proto_paren,      // space between 'func' and '(' - "foo (" vs "foo("
    UO_sp_func_class_paren,      // space between ctor/dtor and '('
 
    UO_sp_attribute_paren,       // space between '__attribute__' and '('
    UO_sp_defined_paren,
+   UO_sp_throw_paren,
 
    UO_sp_type_func,             // space between return type and 'func'
    // a minimum of 1 is forced except for '*'
    UO_sp_before_ptr_star,       // space before a '*' that is part of a type
    UO_sp_before_unnamed_ptr_star,
    UO_sp_after_ptr_star,        // space after a '*' that is part of a type
+   UO_sp_after_ptr_star_func,   // space between a '*' and a function proto/def
+   UO_sp_before_ptr_star_func,
+   UO_sp_after_byref_func,
+   UO_sp_before_byref_func,
    UO_sp_between_ptr_star,      // space between two '*' that are part of a type
 
    UO_sp_special_semi,      /* space empty stmt ';' on while, if, for
@@ -241,6 +261,7 @@ enum uncrustify_options
    UO_sp_after_operator_sym,    // space after operator when followed by a punctuator
    UO_sp_else_brace,
    UO_sp_brace_else,
+   UO_sp_brace_typedef,
 
    UO_sp_catch_brace,
    UO_sp_brace_catch,
@@ -262,9 +283,15 @@ enum uncrustify_options
    UO_sp_before_nl_cont,
    UO_sp_after_oc_scope,
    UO_sp_before_oc_colon,
+   UO_sp_after_oc_colon,
+   UO_sp_before_send_oc_colon,
+   UO_sp_after_send_oc_colon,
    UO_sp_after_oc_type,
    UO_sp_cond_colon,
    UO_sp_cond_question,
+   UO_sp_case_label,
+   UO_sp_range,
+   UO_sp_cmt_cpp_start,
 
    /*
     * Line splitting options (for long lines)
@@ -316,6 +343,7 @@ enum uncrustify_options
    UO_align_struct_init_span,     // align structure initializer values
    UO_align_func_proto_span,      // align function prototypes
    UO_align_func_proto_gap,       // align function prototypes
+   UO_align_on_operator,
    UO_align_mix_var_proto,        // mix function prototypes and variable decl
    UO_align_single_line_func,     // mix single line function with prototypes
    UO_align_single_line_brace,    // align the open brace of single line functions
@@ -331,6 +359,7 @@ enum uncrustify_options
                                   // 2: '*' part of type, dangling
    UO_align_typedef_amp_style,    // align_typedef_star_style for ref '&' stuff
    //UO_align_struct_array_brace,   // TODO: align array of structure initializers
+   UO_align_left_shift,
 
 
    /*
@@ -343,6 +372,7 @@ enum uncrustify_options
    UO_nl_func_decl_args,          // newline after each ',' in a function decl
    UO_nl_func_decl_end,           // newline before the ')' in a function decl
    UO_nl_func_type_name,          // newline between return type and func name in def
+   UO_nl_func_scope_name,
    UO_nl_func_proto_type_name,    // nl_func_type_name, but for prottypes
    UO_nl_func_var_def_blk,        // newline after a block of variable defs
    UO_nl_before_case,             // newline before 'case' statement
@@ -354,6 +384,7 @@ enum uncrustify_options
    UO_nl_after_brace_open,        // force a newline after a brace open
    UO_nl_after_brace_open_cmt,    // put the newline before the comment
    UO_nl_after_vbrace_open,       // force a newline after a virtual brace open
+   UO_nl_after_brace_close,       // force a newline after a brace close
    UO_nl_fcall_brace,             /* newline between function call and open brace */
    UO_nl_squeeze_ifdef,           /* no blanks after #ifxx, #elxx, or before #endif */
    UO_nl_enum_brace,              /* nl between enum and brace */
@@ -421,6 +452,8 @@ enum uncrustify_options
    UO_nl_create_for_one_liner,
    UO_nl_create_while_one_liner,
 
+   UO_pos_arith,                  // position of trailing/leading arithmetic ops
+   UO_pos_assign,                 // position of trailing/leading =
    UO_pos_bool,                   // position of trailing/leading &&/||
    UO_pos_comma,                  // position of comma in functions
    UO_pos_class_comma,            // position of comma in constructor init list
@@ -444,6 +477,10 @@ enum uncrustify_options
    UO_nl_max,                        // maximum consecutive newlines (3 = 2 blank lines)
    UO_nl_before_access_spec,         // number of newlines before "private:", "public:" (0=no change)
    UO_nl_after_access_spec,          // number of newlines after "private:", "public:" (0=no change)
+   UO_nl_comment_func_def,
+   UO_nl_after_try_catch_finally,
+   UO_nl_between_get_set,
+   UO_nl_around_cs_property,
 
    UO_eat_blanks_after_open_brace,   // remove blank lines after {
    UO_eat_blanks_before_close_brace, // remove blank lines before }
@@ -469,6 +506,7 @@ enum uncrustify_options
    UO_mod_sort_using,
    UO_mod_sort_include,
    UO_mod_move_case_break,
+   UO_mod_remove_empty_return,
 
 
    /*
@@ -490,8 +528,10 @@ enum uncrustify_options
    UO_cmt_multi_check_last,
 
    UO_cmt_insert_file_header,
+   UO_cmt_insert_file_footer,
    UO_cmt_insert_func_header,
    UO_cmt_insert_class_header,
+   UO_cmt_insert_before_preproc,
 
    UO_string_escape_char,       // the string escape char to use
    UO_string_escape_char2,      // the string escape char to use
