@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: output.cpp 1290 2008-07-18 00:03:31Z bengardner $
+ * $Id: output.cpp 1469 2009-01-28 01:33:24Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -34,11 +34,21 @@ static chunk_t *output_comment_cpp(chunk_t *pc);
 static void add_comment_text(const char *text, int len,
                              cmt_reflow& cmt, bool esc_close);
 
-
+/**
+ * All output text is sent here, one char at a time.
+ */
 static void add_char(char ch)
 {
    static char last_char = 0;
 
+   /* If we did a '\r' and it isn't followed by a '\n', then output a newline */
+   if ((last_char == '\r') && (ch != '\n'))
+   {
+      fputs(cpd.newline, cpd.fout);
+      cpd.column      = 1;
+      cpd.did_newline = 1;
+   }
+
    /* convert a newline into the LF/CRLF/CR sequence */
    if (ch == '\n')
    {
@@ -46,6 +56,12 @@ static void add_char(char ch)
       cpd.column      = 1;
       cpd.did_newline = 1;
    }
+   else if (ch == '\r')
+   {
+      /* do not output '\r' */
+      cpd.column      = 1;
+      cpd.did_newline = 1;
+   }
    else
    {
       /* Explicitly disallow a tab after a space */
@@ -94,6 +110,31 @@ static void add_text_len(const char *text, int len)
    }
 }
 
+/**
+ * Count the number of characters to the end of the next chunk of text.
+ * If it exceeds the limit, return true.
+ */
+static bool next_word_exceeds_limit(const char *text)
+{
+   int length = 0;
+
+   /* Count any whitespace */
+   while ((*text != 0) && unc_isspace(*text))
+   {
+      text++;
+      length++;
+   }
+
+   /* Count non-whitespace */
+   while ((*text != 0) && !unc_isspace(*text))
+   {
+      text++;
+      length++;
+   }
+   return((cpd.column + length - 1) > cpd.settings[UO_cmt_width].n);
+}
+
+
 /**
  * Advance to a specific column
  * cpd.column is the current column
@@ -272,6 +313,11 @@ void output_text(FILE *pfile)
       {
          pc = output_comment_c(pc);
       }
+      else if (pc->type == CT_JUNK)
+      {
+         /* do not adjust the column for junk */
+         add_text_len(pc->str, pc->len);
+      }
       else if (pc->len == 0)
       {
          /* don't do anything for non-visible stuff */
@@ -472,6 +518,7 @@ static chunk_t *get_next_function(chunk_t *pc)
    while ((pc = chunk_get_next(pc)) != NULL)
    {
       if ((pc->type == CT_FUNC_DEF) ||
+          (pc->type == CT_OC_MSG_DECL) ||
           (pc->type == CT_FUNC_PROTO))
       {
          return(pc);
@@ -506,7 +553,6 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
    chunk_t *prev;
    bool    has_param = true;
    bool    need_nl   = false;
-   int     col       = cpd.column;
 
    fpo = chunk_get_next_type(pc, CT_FPAREN_OPEN, pc->level);
    if (fpo == NULL)
@@ -544,8 +590,7 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
          {
             if (need_nl)
             {
-               add_comment_text("\n", 1, cmt, false);
-               output_to_column(col, false);
+               add_comment_text("\n ", 2, cmt, false);
             }
             need_nl = true;
             add_text("@param");
@@ -574,8 +619,7 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
    {
       if (need_nl)
       {
-         add_comment_text("\n", 1, cmt, false);
-         output_to_column(col, false);
+         add_comment_text("\n ", 2, cmt, false);
       }
       add_text("@return TODO");
    }
@@ -612,6 +656,10 @@ static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
 
    if ((len >= 11) && (memcmp(text, "$(function)", 11) == 0))
    {
+      if (fcn->parent_type == CT_OPERATOR)
+      {
+         add_text_len("operator ", 9);
+      }
       add_text_len(fcn->str, fcn->len);
       return(11);
    }
@@ -623,6 +671,10 @@ static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
    if ((len >= 9) && (memcmp(text, "$(fclass)", 9) == 0))
    {
       chunk_t *tmp = chunk_get_prev_ncnl(fcn);
+      if ((tmp != NULL) && (tmp->type == CT_OPERATOR))
+      {
+         tmp = chunk_get_prev_ncnl(tmp);
+      }
       if ((tmp != NULL) && ((tmp->type == CT_DC_MEMBER) ||
                             (tmp->type == CT_MEMBER)))
       {
@@ -670,7 +722,8 @@ static void add_comment_text(const char *text, int len,
       if ((text[idx] == '\n') ||
           ((text[idx] == ' ') &&
            (cpd.settings[UO_cmt_width].n > 0) &&
-           (cpd.column > cpd.settings[UO_cmt_width].n)))
+           ((cpd.column > cpd.settings[UO_cmt_width].n) ||
+            next_word_exceeds_limit(text + idx))))
       {
          in_word = false;
          add_char('\n');
@@ -829,8 +882,40 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    /* CPP comments can't be grouped unless they are converted to C comments */
    if (!cpd.settings[UO_cmt_cpp_to_c].b)
    {
-      cmt.cont_text = "// ";
-      add_comment_text(first->str, first->len, cmt, false);
+      cmt.cont_text = (cpd.settings[UO_sp_cmt_cpp_start].a & AV_REMOVE) ? "//" : "// ";
+
+      if (cpd.settings[UO_sp_cmt_cpp_start].a == AV_IGNORE)
+      {
+         add_comment_text(first->str, first->len, cmt, false);
+      }
+      else
+      {
+         add_comment_text(first->str, 2, cmt, false);
+
+         const char *tmp = first->str + 2;
+         int        len = first->len - 2;
+
+         if (cpd.settings[UO_sp_cmt_cpp_start].a & AV_REMOVE)
+         {
+            while ((len > 0) && unc_isspace(*tmp))
+            {
+               len--;
+               tmp++;
+            }
+         }
+         if (len > 0)
+         {
+            if (cpd.settings[UO_sp_cmt_cpp_start].a & AV_ADD)
+            {
+               if (!unc_isspace(*tmp))
+               {
+                  add_comment_text(" ", 1, cmt, false);
+               }
+            }
+            add_comment_text(tmp, len, cmt, false);
+         }
+      }
+
       return(first);
    }
 
@@ -924,10 +1009,12 @@ static void output_comment_multi(chunk_t *pc)
    //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d\n",
    //        pc->orig_line, cmt_col, pc->orig_col, col_diff, xtra);
 
-   ccol      = pc->column;
-   remaining = pc->len;
-   cmt_str   = pc->str;
-   line_len  = 0;
+   ccol          = pc->column;
+   remaining     = pc->len;
+   cmt_str       = pc->str;
+   line_len      = 0;
+   cmt.column    = ccol;
+   cmt.br_column = ccol;
    while (remaining > 0)
    {
       ch = *cmt_str;
