@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: prototypes.h 1294 2008-07-18 01:04:49Z bengardner $
+ * $Id: prototypes.h 1456 2008-12-13 22:24:43Z bengardner $
  */
 #ifndef C_PARSE_PROTOTYPES_H_INCLUDED
 #define C_PARSE_PROTOTYPES_H_INCLUDED
@@ -19,12 +19,20 @@
  */
 
 const char *get_token_name(c_token_t token);
+c_token_t find_token_name(const char *text);
 void log_pcf_flags(log_sev_t sev, UINT32 flags);
 const char *path_basename(const char *path);
 int path_dirname_len(const char *filename);
 const char *get_file_extension(int& idx);
 
 
+/*
+ * detect.cpp
+ */
+
+void detect_options(const char *data, int data_len);
+
+
 /*
  *  output.cpp
  */
@@ -130,6 +138,8 @@ chunk_t *skip_template_prev(chunk_t *ang_close);
 chunk_t *skip_attribute_next(chunk_t *attr);
 chunk_t *skip_attribute_prev(chunk_t *fp_close);
 
+void remove_extra_returns();
+
 
 /*
  *  newlines.cpp
@@ -192,7 +202,7 @@ void brace_cleanup(void);
 
 int load_keyword_file(const char *filename);
 const chunk_tag_t *find_keyword(const char *word, int len);
-void add_keyword(const char *tag, c_token_t type, UINT8 lang_flags);
+void add_keyword(const char *tag, c_token_t type, int lang_flags);
 void output_types(FILE *pfile);
 const chunk_tag_t *get_custom_keyword_idx(int& idx);
 void clear_keyword_file(void);
@@ -215,7 +225,7 @@ void clear_defines(void);
 /*
  *  punctuators.cpp
  */
-const chunk_tag_t *find_punctuator(const char *str, UINT8 lang_flags);
+const chunk_tag_t *find_punctuator(const char *str, int lang_flags);
 
 
 /*
