@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: space.cpp 1299 2008-07-18 21:24:41Z bengardner $
+ * $Id: space.cpp 1490 2009-02-27 03:23:28Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -59,7 +59,6 @@ struct no_space_table_s no_space_table[] =
    { CT_NEG,            CT_UNKNOWN       },
    { CT_UNKNOWN,        CT_SQUARE_OPEN   },
    { CT_UNKNOWN,        CT_SQUARE_CLOSE  },
-   { CT_UNKNOWN,        CT_CASE_COLON    },
    { CT_SQUARE_OPEN,    CT_UNKNOWN       },
    { CT_PAREN_CLOSE,    CT_WORD          },
    { CT_PAREN_CLOSE,    CT_FUNC_DEF      },
@@ -68,7 +67,9 @@ struct no_space_table_s no_space_table[] =
    { CT_PAREN_CLOSE,    CT_FPAREN_OPEN   },
 };
 
-#define log_rule(rule) log_rule2(__LINE__, (rule), first, second, complete)
+#define log_rule(rule) \
+   do { if (log_sev_on(LSPACE)) { log_rule2(__LINE__, (rule), first, second, complete); } } while (0)
+
 static void log_rule2(int line, const char *rule, chunk_t *first, chunk_t *second, bool complete)
 {
    LOG_FMT(LSPACE, "Spacing: line %d [%s/%s] '%.*s' <===> [%s/%s] '%.*s' : %s[%d]%s",
@@ -93,6 +94,7 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
 {
    int      idx;
    argval_t arg;
+   chunk_t  *next;
 
    if ((first->type == CT_SPACE) || (second->type == CT_SPACE))
    {
@@ -139,6 +141,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_d_array_colon].a);
    }
 
+   if ((first->type == CT_CASE) && CharTable::IsKw1(second->str[0]))
+   {
+      log_rule("sp_case_label");
+      return(argval_t(cpd.settings[UO_sp_case_label].a | AV_ADD));
+   }
+
    if ((first->type == CT_QUESTION) || (second->type == CT_QUESTION))
    {
       if (cpd.settings[UO_sp_cond_question].a != AV_IGNORE)
@@ -153,6 +161,10 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
          return(cpd.settings[UO_sp_cond_colon].a);
       }
    }
+   if ((first->type == CT_RANGE) || (second->type == CT_RANGE))
+   {
+      return(cpd.settings[UO_sp_range].a);
+   }
 
    if ((first->type == CT_COLON) && (first->parent_type == CT_SQL_EXEC))
    {
@@ -321,13 +333,51 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_before_sparen].a);
    }
 
-   if ((first->type == CT_ASSIGN) || (second->type == CT_ASSIGN))
+   if ((first->type == CT_LAMBDA) ||
+       (second->type == CT_LAMBDA))
+   {
+      log_rule("sp_assign (lambda)");
+      return(cpd.settings[UO_sp_assign].a);
+   }
+
+   if (second->type == CT_ASSIGN)
+   {
+      if (second->flags & PCF_IN_ENUM)
+      {
+         if (cpd.settings[UO_sp_enum_before_assign].a != AV_IGNORE)
+         {
+            log_rule("sp_enum_before_assign");
+            return(cpd.settings[UO_sp_enum_before_assign].a);
+         }
+         log_rule("sp_enum_assign");
+         return(cpd.settings[UO_sp_enum_assign].a);
+      }
+      if (cpd.settings[UO_sp_before_assign].a != AV_IGNORE)
+      {
+         log_rule("sp_before_assign");
+         return(cpd.settings[UO_sp_before_assign].a);
+      }
+      log_rule("sp_assign");
+      return(cpd.settings[UO_sp_assign].a);
+   }
+
+   if (first->type == CT_ASSIGN)
    {
       if (first->flags & PCF_IN_ENUM)
       {
+         if (cpd.settings[UO_sp_enum_after_assign].a != AV_IGNORE)
+         {
+            log_rule("sp_enum_after_assign");
+            return(cpd.settings[UO_sp_enum_after_assign].a);
+         }
          log_rule("sp_enum_assign");
          return(cpd.settings[UO_sp_enum_assign].a);
       }
+      if (cpd.settings[UO_sp_after_assign].a != AV_IGNORE)
+      {
+         log_rule("sp_after_assign");
+         return(cpd.settings[UO_sp_after_assign].a);
+      }
       log_rule("sp_assign");
       return(cpd.settings[UO_sp_assign].a);
    }
@@ -393,6 +443,15 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       }
    }
 
+   if ((first->type == CT_BYREF) &&
+       (cpd.settings[UO_sp_after_byref_func].a != AV_IGNORE) &&
+       ((first->parent_type == CT_FUNC_DEF) ||
+        (first->parent_type == CT_FUNC_PROTO)))
+   {
+      log_rule("sp_after_byref_func");
+      return(cpd.settings[UO_sp_after_byref_func].a);
+   }
+
    if ((first->type == CT_BYREF) && CharTable::IsKw1(second->str[0]))
    {
       log_rule("sp_after_byref");
@@ -401,9 +460,20 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
 
    if (second->type == CT_BYREF)
    {
+      if (cpd.settings[UO_sp_before_byref_func].a != AV_IGNORE)
+      {
+         next = chunk_get_next(second);
+         if ((next != NULL) &&
+             ((next->type == CT_FUNC_DEF) ||
+              (next->type == CT_FUNC_PROTO)))
+         {
+            return(cpd.settings[UO_sp_before_byref_func].a);
+         }
+      }
+
       if (cpd.settings[UO_sp_before_unnamed_byref].a != AV_IGNORE)
       {
-         chunk_t *next = chunk_get_next_nc(second);
+         next = chunk_get_next_nc(second);
          if ((next != NULL) && (next->type != CT_WORD))
          {
             log_rule("sp_before_unnamed_byref");
@@ -414,11 +484,20 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_before_byref].a);
    }
 
-   /* "for (...) {...}" vs "for (...){...}" */
-   if ((first->type == CT_SPAREN_CLOSE) && !chunk_is_comment(second))
+   if (first->type == CT_SPAREN_CLOSE)
    {
-      log_rule("sp_after_sparen");
-      return(cpd.settings[UO_sp_after_sparen].a);
+      if ((second->type == CT_BRACE_OPEN) &&
+          (cpd.settings[UO_sp_sparen_brace].a != AV_IGNORE))
+      {
+         log_rule("sp_sparen_brace");
+         return(cpd.settings[UO_sp_sparen_brace].a);
+      }
+      if (!chunk_is_comment(second) &&
+          (cpd.settings[UO_sp_after_sparen].a != AV_IGNORE))
+      {
+         log_rule("sp_after_sparen");
+         return(cpd.settings[UO_sp_after_sparen].a);
+      }
    }
 
    if ((second->type == CT_FPAREN_OPEN) &&
@@ -436,17 +515,36 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       log_rule("sp_func_call_paren");
       return(cpd.settings[UO_sp_func_call_paren].a);
    }
+   if (first->type == CT_FUNC_CALL_USER)
+   {
+      log_rule("sp_func_call_user_paren");
+      return(cpd.settings[UO_sp_func_call_user_paren].a);
+   }
    if (first->type == CT_ATTRIBUTE)
    {
       log_rule("sp_attribute_paren");
       return(cpd.settings[UO_sp_attribute_paren].a);
    }
-   if ((first->type == CT_FUNC_DEF) || (first->type == CT_CPP_CAST))
+   if (first->type == CT_FUNC_DEF)
    {
-      /* TODO: create new option UO_sp_cpp_cast_paren */
       log_rule("sp_func_def_paren");
       return(cpd.settings[UO_sp_func_def_paren].a);
    }
+   if (first->type == CT_CPP_CAST)
+   {
+      log_rule("sp_cpp_cast_paren");
+      return(cpd.settings[UO_sp_cpp_cast_paren].a);
+   }
+
+   if ((first->type == CT_PAREN_CLOSE) &&
+       ((second->type == CT_PAREN_OPEN) ||
+        (second->type == CT_FPAREN_OPEN)))
+   {
+      /* Must be an indirect/chained function call? */
+      log_rule("REMOVE");
+      return(AV_REMOVE);  /* TODO: make this configurable? */
+   }
+
    if ((first->type == CT_FUNC_PROTO) ||
        ((second->type == CT_FPAREN_OPEN) &&
         (second->parent_type == CT_FUNC_PROTO)))
@@ -500,6 +598,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_defined_paren].a);
    }
 
+   if ((first->type == CT_THROW) && (second->type == CT_PAREN_OPEN))
+   {
+      log_rule("sp_throw_paren");
+      return(cpd.settings[UO_sp_throw_paren].a);
+   }
+
    if ((first->type == CT_THIS) && (second->type == CT_PAREN_OPEN))
    {
       log_rule("REMOVE");
@@ -542,12 +646,6 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_fparen_brace].a);
    }
 
-   if ((first->type == CT_SPAREN_CLOSE) && (second->type == CT_BRACE_OPEN))
-   {
-      log_rule("sp_sparen_brace");
-      return(cpd.settings[UO_sp_sparen_brace].a);
-   }
-
    if ((first->type == CT_D_TEMPLATE) || (second->type == CT_D_TEMPLATE))
    {
       log_rule("REMOVE");
@@ -590,6 +688,13 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_getset_brace].a);
    }
 
+   if ((second->type == CT_PAREN_OPEN) &&
+       (second->parent_type == CT_INVARIANT))
+   {
+      log_rule("sp_invariant_paren");
+      return(cpd.settings[UO_sp_invariant_paren].a);
+   }
+
    if (first->type == CT_PAREN_CLOSE)
    {
       if (first->parent_type == CT_D_TEMPLATE)
@@ -598,6 +703,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
          return(AV_FORCE);
       }
 
+      if (first->parent_type == CT_INVARIANT)
+      {
+         log_rule("sp_after_invariant_paren");
+         return(cpd.settings[UO_sp_after_invariant_paren].a);
+      }
+
       /* Arith after a cast comes first */
       if (second->type == CT_ARITH)
       {
@@ -711,6 +822,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
    }
 
    /* "if(...)" vs "if( ... )" */
+   if ((second->type == CT_SPAREN_CLOSE) &&
+       (cpd.settings[UO_sp_inside_sparen_close].a != AV_IGNORE))
+   {
+      log_rule("sp_inside_sparen_close");
+      return(cpd.settings[UO_sp_inside_sparen_close].a);
+   }
    if ((first->type == CT_SPAREN_OPEN) || (second->type == CT_SPAREN_CLOSE))
    {
       log_rule("sp_inside_sparen");
@@ -729,6 +846,12 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       log_rule("sp_before_class_colon");
       return(cpd.settings[UO_sp_before_class_colon].a);
    }
+   if ((cpd.settings[UO_sp_before_case_colon].a != AV_IGNORE) &&
+       (second->type == CT_CASE_COLON))
+   {
+      log_rule("sp_before_case_colon");
+      return(cpd.settings[UO_sp_before_case_colon].a);
+   }
 
    if (first->type == CT_DOT)
    {
@@ -778,6 +901,15 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_between_ptr_star].a);
    }
 
+   if ((first->type == CT_PTR_TYPE) &&
+       (cpd.settings[UO_sp_after_ptr_star_func].a != AV_IGNORE) &&
+       ((first->parent_type == CT_FUNC_DEF) ||
+        (first->parent_type == CT_FUNC_PROTO)))
+   {
+      log_rule("sp_after_ptr_star_func");
+      return(cpd.settings[UO_sp_after_ptr_star_func].a);
+   }
+
    if ((first->type == CT_PTR_TYPE) &&
        (cpd.settings[UO_sp_after_ptr_star].a != AV_IGNORE) &&
        CharTable::IsKw1(second->str[0]))
@@ -788,9 +920,30 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
 
    if (second->type == CT_PTR_TYPE)
    {
+      if (cpd.settings[UO_sp_before_ptr_star_func].a != AV_IGNORE)
+      {
+         /* Find the next non-'*' chunk */
+         next = second;
+         do
+         {
+            next = chunk_get_next(next);
+         } while ((next != NULL) && (next->type == CT_PTR_TYPE));
+
+         if ((next != NULL) &&
+             ((next->type == CT_FUNC_DEF) ||
+              (next->type == CT_FUNC_PROTO)))
+         {
+            return(cpd.settings[UO_sp_before_ptr_star_func].a);
+         }
+      }
+
       if (cpd.settings[UO_sp_before_unnamed_ptr_star].a != AV_IGNORE)
       {
-         chunk_t *next = chunk_get_next_nc(second);
+         next = chunk_get_next_nc(second);
+         while ((next != NULL) && (next->type == CT_PTR_TYPE))
+         {
+            next = chunk_get_next_nc(next);
+         }
          if ((next != NULL) && (next->type != CT_WORD))
          {
             log_rule("sp_before_unnamed_ptr_star");
@@ -821,7 +974,7 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_type_func].a);
    }
 
-   if (first->type == CT_BRACE_OPEN)
+   if (first->type == CT_BRACE_CLOSE)
    {
       if (second->type == CT_ELSE)
       {
@@ -873,12 +1026,14 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_inside_braces].a);
    }
 
-   if ((first->type == CT_PAREN_CLOSE) &&
-       (second->type == CT_PAREN_OPEN))
+   if ((first->type == CT_BRACE_CLOSE) &&
+       (first->flags & PCF_IN_TYPEDEF) &&
+       ((first->parent_type == CT_ENUM) ||
+        (first->parent_type == CT_STRUCT) ||
+        (first->parent_type == CT_UNION)))
    {
-      /* Must be an indirect/chained function call? */
-      log_rule("REMOVE");
-      return(AV_REMOVE);  /* TODO: make this configurable? */
+      log_rule("sp_brace_typedef");
+      return(cpd.settings[UO_sp_brace_typedef].a);
    }
 
    if (second->type == CT_SPAREN_OPEN)
@@ -887,14 +1042,6 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       return(cpd.settings[UO_sp_before_sparen].a);
    }
 
-   if ((first->type == CT_SPAREN_CLOSE) &&
-       (second->type == CT_SEMICOLON) &&
-       (first->parent_type == CT_WHILE_OF_DO))
-   {
-      log_rule("REMOVE");
-      return(AV_REMOVE); /*TODO: does this need to be configured? */
-   }
-
    if ((second->type != CT_PTR_TYPE) &&
        ((first->type == CT_QUALIFIER) || (first->type == CT_TYPE)))
    {
@@ -953,15 +1100,46 @@ argval_t do_space(chunk_t *first, chunk_t *second, bool complete=true)
       log_rule("sp_incdec");
       return(cpd.settings[UO_sp_incdec].a);
    }
+   if (second->type == CT_CS_SQ_COLON)
+   {
+      log_rule("REMOVE");
+      return(AV_REMOVE);
+   }
+   if (first->type == CT_CS_SQ_COLON)
+   {
+      log_rule("FORCE");
+      return(AV_FORCE);
+   }
    if (first->type == CT_OC_SCOPE)
    {
       log_rule("sp_after_oc_scope");
       return(cpd.settings[UO_sp_after_oc_scope].a);
    }
+   if (first->type == CT_OC_COLON)
+   {
+      if (first->parent_type == CT_OC_MSG)
+      {
+         log_rule("sp_after_send_oc_colon");
+         return(cpd.settings[UO_sp_after_send_oc_colon].a);
+      }
+      else
+      {
+         log_rule("sp_after_oc_colon");
+         return(cpd.settings[UO_sp_after_oc_colon].a);
+      }
+   }
    if (second->type == CT_OC_COLON)
    {
-      log_rule("sp_before_oc_scope");
-      return(cpd.settings[UO_sp_before_oc_colon].a);
+      if (first->parent_type == CT_OC_MSG)
+      {
+         log_rule("sp_before_send_oc_colon");
+         return(cpd.settings[UO_sp_before_send_oc_colon].a);
+      }
+      else
+      {
+         log_rule("sp_before_oc_colon");
+         return(cpd.settings[UO_sp_before_oc_colon].a);
+      }
    }
 
    if ((second->type == CT_COMMENT) && (second->parent_type == CT_COMMENT_EMBED))
