@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: token_enum.h 1286 2008-07-17 23:16:05Z bengardner $
+ * $Id: token_enum.h 1456 2008-12-13 22:24:43Z bengardner $
  */
 #ifndef TOKEN_ENUM_H_INCLUDED
 #define TOKEN_ENUM_H_INCLUDED
@@ -25,6 +25,7 @@ typedef enum
    CT_EOF,
    CT_UNKNOWN,
 
+   CT_JUNK,          /* junk collected when parsing is disabled */
 
    CT_WHITESPACE,    /* whitespace without any newlines */
    CT_SPACE,         /* a fixed number of spaces to support weird spacing rules */
@@ -139,6 +140,7 @@ typedef enum
    CT_DELEGATE,
    CT_BODY,
    CT_DEBUG,
+   CT_DEBUGGER,
    CT_INVARIANT,
    CT_UNITTEST,
    CT_UNSAFE,
@@ -181,6 +183,7 @@ typedef enum
    CT_LABEL_COLON,      /* the colon for a label */
    CT_FUNCTION,         /* function - unspecified, call mark_function() */
    CT_FUNC_CALL,        /* function call */
+   CT_FUNC_CALL_USER,   /* function call (special user) */
    CT_FUNC_DEF,         /* function definition/implementation */
    CT_FUNC_PROTO,       /* function prototype */
    CT_FUNC_CLASS,       /* ctor or dtor for a class */
@@ -195,11 +198,14 @@ typedef enum
 
    CT_BIT_COLON,        /* a ':' in a variable declaration */
 
+   CT_OC_DYNAMIC,
    CT_OC_END,           /* ObjC: @end */
    CT_OC_IMPL,          /* ObjC: @implementation */
    CT_OC_INTF,          /* ObjC: @interface */
+   CT_OC_PROPERTY,      /* ObjC: @property */
    CT_OC_CLASS,         /* ObjC: the name after @interface or @implementation */
    CT_OC_SCOPE,         /* ObjC: the '-' or '+' in '-(void) func: (int) i;' */
+   CT_OC_MSG,           /* ObjC: parent type to '[class func : param];' stuff */
    CT_OC_MSG_SPEC,      /* ObjC: msg spec '-(void) func: (int) i;' */
    CT_OC_MSG_DECL,      /* ObjC: msg decl '-(void) func: (int) i { }' */
    CT_OC_COLON,         /* ObjC: the colon in a msg spec */
@@ -252,7 +258,11 @@ typedef enum
    CT_BASE,             /* C# thingy */
    CT_DEFAULT,          /* may be changed into CT_CASE */
    CT_GETSET,           /* must be followed by CT_BRACE_OPEN or reverts to CT_WORD */
+   CT_GETSET_EMPTY,     /* get/set/add/remove followed by a semicolon */
    CT_CONCAT,           /* The '~' between strings */
+   CT_CS_SQ_STMT,       /* '[assembly: xxx]' or '[Attribute()]' or '[Help()]', etc */
+   CT_CS_SQ_COLON,      /* the colon in one of those [] thingys */
+   CT_CS_PROPERTY,      /* word or ']' followed by '{' */
 
    /* Embedded SQL - always terminated with a semicolon */
    CT_SQL_EXEC,         /* the 'EXEC' in 'EXEC SQL ...' */
