@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: tokenize.cpp 1232 2008-06-14 22:00:39Z bengardner $
+ * $Id: tokenize.cpp 1475 2009-02-07 21:23:02Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -60,6 +60,7 @@ static bool parse_comment(chunk_t *pc)
    int  len     = 2;
    bool is_d    = (cpd.lang_flags & LANG_D) != 0;
    int  d_level = 0;
+   int  bs_cnt;
 
    if ((pc->str[0] != '/') ||
        ((pc->str[1] != '*') && (pc->str[1] != '/') &&
@@ -76,18 +77,37 @@ static bool parse_comment(chunk_t *pc)
       pc->type = CT_COMMENT_CPP;
       while (true)
       {
+         bs_cnt = 0;
          while ((pc->str[len] != '\n') &&
                 (pc->str[len] != '\r') &&
                 (pc->str[len] != 0))
          {
+            if (pc->str[len] == '\\')
+            {
+               bs_cnt++;
+            }
+            else
+            {
+               bs_cnt = 0;
+            }
             len++;
          }
 
-         if (pc->str[len - 1] != '\\')
+         /* If we hit an odd number of backslashes right before the newline,
+          * then we keep going.
+          */
+         if (((bs_cnt & 1) == 0) || (pc->str[len] == 0))
          {
             break;
          }
-         len++;
+         if (pc->str[len] == '\r')
+         {
+            len++;
+         }
+         if (pc->str[len] == '\n')
+         {
+            len++;
+         }
       }
    }
    else if (pc->str[len] == 0)
@@ -154,7 +174,12 @@ static bool parse_comment(chunk_t *pc)
          {
             len        += 2;
             cpd.column += 2;
-            break;
+
+            /* If there is another C comment right after this one, combine them */
+            if ((pc->str[len] != '/') || (pc->str[len + 1] != '*'))
+            {
+               break;
+            }
          }
 
          if ((pc->str[len] == '\n') || (pc->str[len] == '\r'))
@@ -190,6 +215,7 @@ static bool parse_comment(chunk_t *pc)
    {
       if (str_search(UNCRUSTIFY_ON_TEXT, pc->str, pc->len) != NULL)
       {
+         LOG_FMT(LBCTRL, "Found '%s' on line %d\n", UNCRUSTIFY_ON_TEXT, pc->orig_line);
          cpd.unc_off = false;
       }
    }
@@ -197,6 +223,7 @@ static bool parse_comment(chunk_t *pc)
    {
       if (str_search(UNCRUSTIFY_OFF_TEXT, pc->str, pc->len) != NULL)
       {
+         LOG_FMT(LBCTRL, "Found '%s' on line %d\n", UNCRUSTIFY_OFF_TEXT, pc->orig_line);
          cpd.unc_off = true;
       }
    }
@@ -335,7 +362,7 @@ static bool parse_number(chunk_t *pc)
    /* Check the suffixes
     * Valid suffixes per language (not that it matters):
     *        Integer       Float
-    * C/C++: uUlL          lLfF
+    * C/C++: uUlL64        lLfF
     * C#:    uUlL          fFdDMm
     * D:     uUL           ifFL
     * Java:  lL            fFdD
@@ -357,6 +384,20 @@ static bool parse_number(chunk_t *pc)
       len++;
    }
 
+   /* skip the Microsoft-specific '64' suffix */
+   if ((pc->str[len] == '6') && (pc->str[len + 1] == '4'))
+   {
+      len += 2;
+   }
+
+   /* It there is anything left, then we are probably dealing with garbage or
+    * some sick macro junk. Eat it.
+    */
+   while (unc_isalnum(pc->str[len]))
+   {
+      len++;
+   }
+
    pc->len     = len;
    pc->type    = is_float ? CT_NUMBER_FP : CT_NUMBER;
    cpd.column += len;
@@ -384,11 +425,12 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
    end_ch = CharTable::Get(pc->str[len]) & 0xff;
    len++;
 
+   cpd.column += len;
    for ( /* nada */; pc->str[len] != 0; len++)
    {
+      cpd.column++;
       if (!escaped)
       {
-         cpd.column++;
          if (pc->str[len] == escape_char)
          {
             escaped = (escape_char != 0);
@@ -424,6 +466,7 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
         (pc->str[len] == 'd')))
    {
       len++;
+      cpd.column++;
    }
    pc->len = len;
    return(true);
@@ -1016,7 +1059,7 @@ void tokenize(const char *data, int data_len, chunk_t *ref)
       pc = chunk_add_before(&chunk, ref);
 
       /* A newline marks the end of a preprocessor */
-      if ((pc->type == CT_NEWLINE) || (pc->type == CT_COMMENT_MULTI))
+      if (pc->type == CT_NEWLINE) // || (pc->type == CT_COMMENT_MULTI))
       {
          cpd.in_preproc         = CT_NONE;
          cpd.preproc_ncnl_count = 0;
@@ -1082,3 +1125,21 @@ void tokenize(const char *data, int data_len, chunk_t *ref)
       LOG_FMT(LLINEENDS, "Using CR line endings\n");
    }
 }
+
+/**
+ * A simplistic fixed-sized needle in the fixed-size haystack string search.
+ */
+int str_find(const char *needle, int needle_len,
+             const char *haystack, int haystack_len)
+{
+   int idx;
+
+   for (idx = 0; idx < (haystack_len - needle_len); idx++)
+   {
+      if (memcmp(needle, haystack + idx, needle_len) == 0)
+      {
+         return(idx);
+      }
+   }
+   return(-1);
+}
