@@ -8,7 +8,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: tokenize_cleanup.cpp 1257 2008-06-19 00:21:11Z bengardner $
+ * $Id: tokenize_cleanup.cpp 1480 2009-02-21 03:07:51Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -69,6 +69,16 @@ void tokenize_cleanup(void)
          }
       }
 
+      /**
+       * Change CT_BASE before CT_PAREN_OPEN to CT_WORD.
+       * public myclass() : base() {
+       * }
+       */
+      if ((pc->type == CT_BASE) && (next->type == CT_PAREN_OPEN))
+      {
+         pc->type = CT_WORD;
+      }
+
       /**
        * Change CT_WORD after CT_ENUM, CT_UNION, or CT_STRUCT to CT_TYPE
        * Change CT_WORD before CT_WORD to CT_TYPE
@@ -135,7 +145,14 @@ void tokenize_cleanup(void)
        */
       if ((pc->type == CT_ANGLE_OPEN) && (pc->parent_type != CT_TYPE_CAST))
       {
-         check_template(pc);
+         if (pc->flags & PCF_IN_PREPROC)
+         {
+            pc->type = CT_COMPARE;
+         }
+         else
+         {
+            check_template(pc);
+         }
       }
       if ((pc->type == CT_ANGLE_CLOSE) && (pc->parent_type != CT_TEMPLATE))
       {
@@ -182,7 +199,18 @@ void tokenize_cleanup(void)
       /* Change get/set to CT_WORD if not followed by a brace open */
       if ((pc->type == CT_GETSET) && (next->type != CT_BRACE_OPEN))
       {
-         pc->type = CT_WORD;
+         if ((next->type == CT_SEMICOLON) &&
+             ((prev->type == CT_BRACE_CLOSE) ||
+              (prev->type == CT_BRACE_OPEN) ||
+              (prev->type == CT_SEMICOLON)))
+         {
+            pc->type          = CT_GETSET_EMPTY;
+            next->parent_type = CT_GETSET;
+         }
+         else
+         {
+            pc->type = CT_WORD;
+         }
       }
 
       /* Change item after operator (>=, ==, etc) to a CT_FUNCTION */
@@ -322,12 +350,68 @@ void tokenize_cleanup(void)
       /* Detect Objective C class name */
       if ((pc->type == CT_OC_IMPL) || (pc->type == CT_OC_INTF))
       {
-         next->type = CT_OC_CLASS;
-         tmp        = chunk_get_next_ncnl(next);
+         next->type        = CT_OC_CLASS;
+         next->parent_type = pc->type;
+
+         tmp = chunk_get_next_ncnl(next);
          if (tmp != NULL)
          {
             tmp->flags |= PCF_STMT_START | PCF_EXPR_START;
          }
+
+         tmp = chunk_get_next_type(pc, CT_OC_END, pc->level);
+         if (tmp != NULL)
+         {
+            tmp->parent_type = pc->type;
+         }
+      }
+
+      /**
+       * Objective C @dynamic and @synthesize
+       *  @dynamic xxx, yyy;
+       *  @synthesize xxx, yyy;
+       * Just find the semicolon and mark it.
+       */
+      if (pc->type == CT_OC_DYNAMIC)
+      {
+         tmp = chunk_get_next_type(pc, CT_SEMICOLON, pc->level);
+         if (tmp != NULL)
+         {
+            tmp->parent_type = pc->type;
+         }
+      }
+
+      /* Detect Objective C @property
+       *  @property NSString *stringProperty;
+       *  @property(nonatomic, retain) NSMutableDictionary *shareWith;
+       */
+      if (pc->type == CT_OC_PROPERTY)
+      {
+         if (next->type != CT_PAREN_OPEN)
+         {
+            next->flags |= PCF_STMT_START | PCF_EXPR_START;
+         }
+         else
+         {
+            next->parent_type = pc->type;
+
+            tmp = chunk_get_next_type(pc, CT_PAREN_CLOSE, pc->level);
+            if (tmp != NULL)
+            {
+               tmp->parent_type = pc->type;
+               tmp = chunk_get_next_ncnl(tmp);
+               if (tmp != NULL)
+               {
+                  tmp->flags |= PCF_STMT_START | PCF_EXPR_START;
+
+                  tmp = chunk_get_next_type(tmp, CT_SEMICOLON, pc->level);
+                  if (tmp != NULL)
+                  {
+                     tmp->parent_type = pc->type;
+                  }
+               }
+            }
+         }
       }
 
       /* Handle special preprocessor junk */
@@ -348,6 +432,66 @@ void tokenize_cleanup(void)
          }
       }
 
+      /* Check for C# nullable types '?' is in next */
+      if ((cpd.lang_flags & LANG_CS) &&
+          (next->type == CT_QUESTION) &&
+          (next->orig_col == (pc->orig_col + pc->len)))
+      {
+         tmp = chunk_get_next_ncnl(next);
+         if (tmp != NULL)
+         {
+            bool doit = ((tmp->type == CT_PAREN_CLOSE) ||
+                         (tmp->type == CT_ANGLE_CLOSE));
+
+            if (tmp->type == CT_WORD)
+            {
+               tmp2 = chunk_get_next_ncnl(tmp);
+               if ((tmp2 != NULL) &&
+                   ((tmp2->type == CT_SEMICOLON) ||
+                    (tmp2->type == CT_ASSIGN) ||
+                    (tmp2->type == CT_COMMA)))
+               {
+                  doit = true;
+               }
+            }
+
+            if (doit)
+            {
+               pc->len++;
+               chunk_del(next);
+               next = tmp;
+            }
+         }
+      }
+
+      /* Convert '>' + '>' into '>>' */
+      if ((cpd.lang_flags & LANG_CS) &&
+          (pc->type == CT_ANGLE_CLOSE) &&
+          (next->type == CT_ANGLE_CLOSE) &&
+          (pc->parent_type == CT_NONE) &&
+          ((pc->orig_col + pc->len) == next->orig_col) &&
+          (next->parent_type == CT_NONE))
+      {
+         pc->len++;
+         pc->type = CT_ARITH;
+         tmp = chunk_get_next_ncnl(next);
+         chunk_del(next);
+         next = tmp;
+      }
+
+      /* Change 'default(' into a sizeof-like statement */
+      if ((cpd.lang_flags & LANG_CS) &&
+          (pc->type == CT_DEFAULT) &&
+          (next->type == CT_PAREN_OPEN))
+      {
+         pc->type = CT_SIZEOF;
+      }
+
+      if ((pc->type == CT_UNSAFE) && (next->type != CT_BRACE_OPEN))
+      {
+         pc->type = CT_QUALIFIER;
+      }
+
       /* TODO: determine other stuff here */
 
       prev = pc;
