@@ -8,7 +8,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: unc_ctype.h 1191 2008-06-06 17:55:28Z bengardner $
+ * $Id: unc_ctype.h 1317 2008-08-04 01:06:49Z bengardner $
  */
 #ifndef UNC_CTYPE_H_INCLUDED
 #define UNC_CTYPE_H_INCLUDED
@@ -35,6 +35,10 @@ static inline int unc_isalpha(int ch)
 {
    return(isalpha(unc_fix_ctype(ch)));
 }
+static inline int unc_isalnum(int ch)
+{
+   return(isalnum(unc_fix_ctype(ch)));
+}
 static inline int unc_toupper(int ch)
 {
    return(toupper(unc_fix_ctype(ch)));
