@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: uncrustify.cpp 1164 2008-04-26 04:49:27Z bengardner $
+ * $Id: uncrustify.cpp 1495 2009-02-28 17:37:10Z bengardner $
  */
 #define DEFINE_PCF_NAMES
 #define DEFINE_CHAR_TABLE
@@ -42,6 +42,8 @@ static int language_from_tag(const char *tag);
 static int language_from_filename(const char *filename);
 static const char *language_to_string(int lang);
 static char *read_stdin(int& out_len);
+static void uncrustify_start(const char *data, int data_len);
+static void uncrustify_end();
 static void uncrustify_file(const char *data, int data_len, FILE *pfout,
                             const char *parsed_file);
 static void do_source_file(const char *filename_in,
@@ -57,6 +59,8 @@ static const char *make_output_filename(char *buf, int buf_size,
                                         const char *prefix,
                                         const char *suffix);
 
+static int load_mem_file(const char *filename, file_mem& fm);
+
 
 /**
  * Replace the brain-dead and non-portable basename().
@@ -147,7 +151,7 @@ static void usage_exit(const char *msg, const char *argv0, int code)
 #ifdef HAVE_UTIME_H
            " --mtime      : preserve mtime on replaced files\n"
 #endif
-           " -l           : language override: C, CPP, D, CS, JAVA, PAWN\n"
+           " -l           : language override: C, CPP, D, CS, JAVA, PAWN, OC, OC+\n"
            " -t           : load a file with types (usually not needed)\n"
            " -q           : quiet mode - no output on stderr (-L will override)\n"
            "\n"
@@ -158,6 +162,8 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            " --update-config          : Output a new config file. Use with -o FILE\n"
            " --update-config-with-doc : Output a new config file. Use with -o FILE\n"
            " --universalindent        : Output a config file for Universal Indent GUI\n"
+           " --detect                 : detects the config from a source file. Use with '-f FILE'\n"
+           "                            Detection is fairly limited.\n"
            "\n"
            "Debug Options:\n"
            " -p FILE      : dump debug info to a file\n"
@@ -175,8 +181,14 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            "uncrustify -c my.cfg --no-backup foo.d\n"
            "uncrustify -c my.cfg --prefix=out -F files.txt\n"
            "\n"
+           "Note: Use comments containing ' *INDENT-OFF*' and ' *INDENT-ON*' to disable\n"
+           "      processing of parts of the source file.\n"
+           "\n"
+           "There are currently %d options and minimal documentation.\n"
+           "Try UniversalIndentGUI and good luck.\n"
+           "\n"
            ,
-           path_basename(argv0));
+           path_basename(argv0), UO_option_count);
    exit(code);
 }
 
@@ -375,6 +387,7 @@ int main(int argc, char *argv[])
    bool keep_mtime       = arg.Present("--mtime");
    bool update_config    = arg.Present("--update-config");
    bool update_config_wd = arg.Present("--update-config-with-doc");
+   bool detect           = arg.Present("--detect");
 
    /* Grab the output override */
    output_file = arg.Param("-o");
@@ -386,6 +399,7 @@ int main(int argc, char *argv[])
    LOG_FMT(LDATA, "suffix      = %s\n", (suffix != NULL) ? suffix : "null");
    LOG_FMT(LDATA, "replace     = %d\n", replace);
    LOG_FMT(LDATA, "no_backup   = %d\n", no_backup);
+   LOG_FMT(LDATA, "detect      = %d\n", detect);
 
    if (replace || no_backup)
    {
@@ -407,7 +421,8 @@ int main(int argc, char *argv[])
    }
 
    /* Try to load the config file, if available.
-    * It is optional for "--universalindent", but required for everything else.
+    * It is optional for "--universalindent" and "--detect", but required for
+    * everything else.
     */
    if (cfg_file != NULL)
    {
@@ -438,6 +453,40 @@ int main(int argc, char *argv[])
       return(EXIT_SUCCESS);
    }
 
+   if (detect)
+   {
+      file_mem fm;
+
+      if ((source_file == NULL) || (source_list != NULL))
+      {
+         fprintf(stderr, "The --detect option requires a single input file\n");
+         return(EXIT_FAILURE);
+      }
+
+      /* Do some simple language detection based on the filename extension */
+      if (cpd.lang_flags == 0)
+      {
+         cpd.lang_flags = language_from_filename(source_file);
+      }
+
+      /* Try to read in the source file */
+      if (load_mem_file(source_file, fm) < 0)
+      {
+         LOG_FMT(LERR, "Failed to load (%s)\n", source_file);
+         cpd.error_count++;
+         return(EXIT_FAILURE);
+      }
+
+      uncrustify_start(fm.data, fm.length);
+      detect_options(fm.data, fm.length);
+      uncrustify_end();
+      free(fm.data);
+
+      redir_stdout(output_file);
+      save_option_file(stdout, update_config_wd);
+      return(EXIT_SUCCESS);
+   }
+
    /* Everything beyond this point requires a config file, so complain and
     * bail if we don't have one.
     */
@@ -675,7 +724,7 @@ static void make_folders(const char *filename)
 /**
  * Loads a file into memory
  */
-int load_mem_file(const char *filename, file_mem& fm)
+static int load_mem_file(const char *filename, file_mem& fm)
 {
    int         retval = -1;
    struct stat my_stat;
@@ -765,6 +814,12 @@ static int load_header_files()
       retval |= load_mem_file_config(cpd.settings[UO_cmt_insert_file_header].str,
                                      cpd.file_hdr);
    }
+   if ((cpd.settings[UO_cmt_insert_file_footer].str != NULL) &&
+       (cpd.settings[UO_cmt_insert_file_footer].str[0] != 0))
+   {
+      retval |= load_mem_file_config(cpd.settings[UO_cmt_insert_file_footer].str,
+                                     cpd.file_ftr);
+   }
    if ((cpd.settings[UO_cmt_insert_func_header].str != NULL) &&
        (cpd.settings[UO_cmt_insert_func_header].str[0] != 0))
    {
@@ -903,10 +958,34 @@ static void add_file_header()
    }
 }
 
+static void add_file_footer()
+{
+   chunk_t *pc = chunk_get_tail();
+
+   /* Back up if the file ends with a newline */
+   if ((pc != NULL) && chunk_is_newline(pc))
+   {
+      pc = chunk_get_prev(pc);
+   }
+   if ((pc != NULL) &&
+       (!chunk_is_comment(pc) || !chunk_is_newline(chunk_get_prev(pc))))
+   {
+      pc = chunk_get_tail();
+      if (!chunk_is_newline(pc))
+      {
+         LOG_FMT(LSYS, "Adding a newline at the end of the file\n");
+         newline_add_after(pc);
+      }
+      tokenize(cpd.file_ftr.data, cpd.file_ftr.length, NULL);
+   }
+}
+
 static void add_func_header(c_token_t type, file_mem& fm)
 {
    chunk_t *pc;
    chunk_t *ref;
+   chunk_t *tmp;
+   bool    do_insert;
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnlnp(pc))
    {
@@ -915,15 +994,37 @@ static void add_func_header(c_token_t type, file_mem& fm)
          continue;
       }
 
+      do_insert = false;
       /* On a function proto or def. Back up to a close brace or semicolon on
        * the same level
        */
       ref = pc;
       while ((ref = chunk_get_prev(ref)) != NULL)
       {
-         if (chunk_is_comment(ref) ||
-             ((ref->level != pc->level) &&
-              (ref->flags & PCF_IN_PREPROC)))
+         /* Bail if we change level */
+         if (ref->level != pc->level)
+         {
+            do_insert = true;
+            break;
+         }
+
+         /* Bail if we hit a preprocessor and cmt_insert_before_preproc is false */
+         if (ref->flags & PCF_IN_PREPROC)
+         {
+            tmp = chunk_get_prev_type(ref, CT_PREPROC, ref->level);
+            if ((tmp != NULL) && (tmp->parent_type == CT_PP_IF))
+            {
+               tmp = chunk_get_prev_nnl(tmp);
+               if (chunk_is_comment(tmp) &&
+                   !cpd.settings[UO_cmt_insert_before_preproc].b)
+               {
+                  break;
+               }
+            }
+         }
+
+         /* Ignore 'right' comments */
+         if (chunk_is_comment(ref) && chunk_is_newline(chunk_get_prev(ref)))
          {
             break;
          }
@@ -933,15 +1034,24 @@ static void add_func_header(c_token_t type, file_mem& fm)
               (ref->type == CT_SEMICOLON) ||
               (ref->type == CT_BRACE_CLOSE)))
          {
-            tokenize(fm.data, fm.length, chunk_get_next_nnl(ref));
+            do_insert = true;
             break;
          }
       }
+      if (do_insert)
+      {
+         /* Insert between after and ref */
+         chunk_t *after = chunk_get_next_ncnl(ref);
+         tokenize(fm.data, fm.length, after);
+         for (tmp = chunk_get_next(ref); tmp != after; tmp = chunk_get_next(tmp))
+         {
+            tmp->level = after->level;
+         }
+      }
    }
 }
 
-static void uncrustify_file(const char *data, int data_len, FILE *pfout,
-                            const char *parsed_file)
+static void uncrustify_start(const char *data, int data_len)
 {
    /**
     * Parse the text into chunks
@@ -954,6 +1064,12 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       add_file_header();
    }
 
+   /* Add the file header */
+   if (cpd.file_ftr.data != NULL)
+   {
+      add_file_footer();
+   }
+
    /**
     * Change certain token types based on simple sequence.
     * Example: change '[' + ']' to '[]'
@@ -982,151 +1098,178 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
     */
    fix_symbols();
 
-   /**
-    * Add comments before function defs and classes
-    */
-   if (cpd.func_hdr.data != NULL)
-   {
-      add_func_header(CT_FUNC_DEF, cpd.func_hdr);
-   }
-   if (cpd.class_hdr.data != NULL)
-   {
-      add_func_header(CT_CLASS, cpd.class_hdr);
-   }
-
-   /* Scrub extra semicolons */
-   if (cpd.settings[UO_mod_remove_extra_semicolon].b)
-   {
-      remove_extra_semicolons();
-   }
-
    /**
     * Look at all colons ':' and mark labels, :? sequences, etc.
     */
    combine_labels();
+}
+
+static void uncrustify_file(const char *data, int data_len, FILE *pfout,
+                            const char *parsed_file)
+{
+   uncrustify_start(data, data_len);
 
    /**
-    * Change virtual braces into real braces...
+    * Done with detection. Do the rest only if the file will go somewhere.
+    * The detection code needs as few changes as possible.
     */
-   do_braces();
-
-   /**
-    * Add parens
-    */
-   do_parens();
-
-   /**
-    * Insert line breaks as needed
-    */
-   do_blank_lines();
-   newlines_cleanup_braces();
-   if (cpd.settings[UO_nl_after_multiline_comment].b)
+   if (pfout != NULL)
    {
-      newline_after_multiline_comment();
-   }
-   newlines_insert_blank_lines();
-   if (cpd.settings[UO_nl_squeeze_ifdef].b)
-   {
-      newlines_squeeze_ifdef();
-   }
-   if (cpd.settings[UO_pos_bool].tp != TP_IGNORE)
-   {
-      newlines_chunk_pos(CT_BOOL, cpd.settings[UO_pos_bool].tp);
-   }
-   if (cpd.settings[UO_pos_comma].tp != TP_IGNORE)
-   {
-      newlines_chunk_pos(CT_COMMA, cpd.settings[UO_pos_comma].tp);
-   }
-   newlines_class_colon_pos();
-   newlines_eat_start_end();
-   newlines_cleanup_dup();
-
-   mark_comments();
-
-   /**
-    * Add balanced spaces around nested params
-    */
-   if (cpd.settings[UO_sp_balance_nested_parens].b)
-   {
-      space_text_balance_nested_parens();
-   }
-
-   /* Scrub certain added semicolons */
-   if (((cpd.lang_flags & LANG_PAWN) != 0) &&
-       cpd.settings[UO_mod_pawn_semicolon].b)
-   {
-      pawn_scrub_vsemi();
-   }
-
-   /* Insert trailing comments after certain close braces */
-   if ((cpd.settings[UO_mod_add_long_switch_closebrace_comment].n > 0) ||
-       (cpd.settings[UO_mod_add_long_function_closebrace_comment].n > 0))
-   {
-      add_long_closebrace_comment();
-   }
-
-   /* Sort imports/using/include */
-   if (cpd.settings[UO_mod_sort_import].b ||
-       cpd.settings[UO_mod_sort_include].b ||
-       cpd.settings[UO_mod_sort_using].b)
-   {
-      sort_imports();
-   }
-
-   /**
-    * Fix same-line inter-chunk spacing
-    */
-   space_text();
-
-   /**
-    * Do any aligning of preprocessors
-    */
-   if (cpd.settings[UO_align_pp_define_span].n > 0)
-   {
-      align_preprocessor();
-   }
-
-   /**
-    * Indent the text
-    */
-   indent_preproc();
-   indent_text();
-
-   /**
-    * Aligning everything else and reindent
-    */
-   align_all();
-   indent_text();
-
-   if (cpd.settings[UO_code_width].n > 0)
-   {
-      int max_passes = 3;
-      int prev_changes;
-      do
+      /**
+       * Add comments before function defs and classes
+       */
+      if (cpd.func_hdr.data != NULL)
       {
-         prev_changes = cpd.changes;
-         do_code_width();
-         if (prev_changes != cpd.changes)
+         add_func_header(CT_FUNC_DEF, cpd.func_hdr);
+      }
+      if (cpd.class_hdr.data != NULL)
+      {
+         add_func_header(CT_CLASS, cpd.class_hdr);
+      }
+
+      /* Scrub extra semicolons */
+      if (cpd.settings[UO_mod_remove_extra_semicolon].b)
+      {
+         remove_extra_semicolons();
+      }
+
+      /* Remove unnecessary returns */
+      if (cpd.settings[UO_mod_remove_empty_return].b)
+      {
+         remove_extra_returns();
+      }
+
+      /**
+       * Change virtual braces into real braces...
+       */
+      do_braces();
+
+      /**
+       * Add parens
+       */
+      do_parens();
+
+      /**
+       * Insert line breaks as needed
+       */
+      do_blank_lines();
+      newlines_cleanup_braces();
+      if (cpd.settings[UO_nl_after_multiline_comment].b)
+      {
+         newline_after_multiline_comment();
+      }
+      newlines_insert_blank_lines();
+      if (cpd.settings[UO_pos_bool].tp != TP_IGNORE)
+      {
+         newlines_chunk_pos(CT_BOOL, cpd.settings[UO_pos_bool].tp);
+      }
+      if (cpd.settings[UO_pos_comma].tp != TP_IGNORE)
+      {
+         newlines_chunk_pos(CT_COMMA, cpd.settings[UO_pos_comma].tp);
+      }
+      if (cpd.settings[UO_pos_assign].tp != TP_IGNORE)
+      {
+         newlines_chunk_pos(CT_ASSIGN, cpd.settings[UO_pos_assign].tp);
+      }
+      if (cpd.settings[UO_pos_arith].tp != TP_IGNORE)
+      {
+         newlines_chunk_pos(CT_ARITH, cpd.settings[UO_pos_arith].tp);
+      }
+      newlines_class_colon_pos();
+      if (cpd.settings[UO_nl_squeeze_ifdef].b)
+      {
+         newlines_squeeze_ifdef();
+      }
+      newlines_eat_start_end();
+      newlines_cleanup_dup();
+
+      mark_comments();
+
+      /**
+       * Add balanced spaces around nested params
+       */
+      if (cpd.settings[UO_sp_balance_nested_parens].b)
+      {
+         space_text_balance_nested_parens();
+      }
+
+      /* Scrub certain added semicolons */
+      if (((cpd.lang_flags & LANG_PAWN) != 0) &&
+          cpd.settings[UO_mod_pawn_semicolon].b)
+      {
+         pawn_scrub_vsemi();
+      }
+
+      /* Sort imports/using/include */
+      if (cpd.settings[UO_mod_sort_import].b ||
+          cpd.settings[UO_mod_sort_include].b ||
+          cpd.settings[UO_mod_sort_using].b)
+      {
+         sort_imports();
+      }
+
+      /**
+       * Fix same-line inter-chunk spacing
+       */
+      space_text();
+
+      /**
+       * Do any aligning of preprocessors
+       */
+      if (cpd.settings[UO_align_pp_define_span].n > 0)
+      {
+         align_preprocessor();
+      }
+
+      /**
+       * Indent the text
+       */
+      indent_preproc();
+      indent_text();
+
+      /* Insert trailing comments after certain close braces */
+      if ((cpd.settings[UO_mod_add_long_switch_closebrace_comment].n > 0) ||
+          (cpd.settings[UO_mod_add_long_function_closebrace_comment].n > 0))
+      {
+         add_long_closebrace_comment();
+      }
+
+      /**
+       * Aligning everything else and reindent
+       */
+      align_all();
+      indent_text();
+
+      if (cpd.settings[UO_code_width].n > 0)
+      {
+         int max_passes = 3;
+         int prev_changes;
+         do
          {
-            align_all();
-            indent_text();
-         }
-      } while ((prev_changes != cpd.changes) && (--max_passes > 0));
-   }
+            prev_changes = cpd.changes;
+            do_code_width();
+            if (prev_changes != cpd.changes)
+            {
+               align_all();
+               indent_text();
+            }
+         } while ((prev_changes != cpd.changes) && (--max_passes > 0));
+      }
 
-   /**
-    * And finally, align the backslash newline stuff
-    */
-   align_right_comments();
-   if (cpd.settings[UO_align_nl_cont].b)
-   {
-      align_backslash_newline();
-   }
+      /**
+       * And finally, align the backslash newline stuff
+       */
+      align_right_comments();
+      if (cpd.settings[UO_align_nl_cont].b)
+      {
+         align_backslash_newline();
+      }
 
-   /**
-    * Now render it all to the output file
-    */
-   output_text(pfout);
+      /**
+       * Now render it all to the output file
+       */
+      output_text(pfout);
+   }
 
    /* Special hook for dumping parsed data for debugging */
    if (parsed_file != NULL)
@@ -1139,6 +1282,11 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       }
    }
 
+   uncrustify_end();
+}
+
+static void uncrustify_end()
+{
    /* Free all the memory */
    chunk_t *pc;
    while ((pc = chunk_get_head()) != NULL)
@@ -1157,6 +1305,27 @@ const char *get_token_name(c_token_t token)
    return("???");
 }
 
+/**
+ * Grab the token id for the text.
+ * returns CT_NONE on failure t match
+ */
+c_token_t find_token_name(const char *text)
+{
+   int idx;
+
+   if ((text != NULL) && (*text != 0))
+   {
+      for (idx = 1; idx < (int)ARRAY_SIZE(token_names); idx++)
+      {
+         if (strcasecmp(text, token_names[idx]) == 0)
+         {
+            return((c_token_t)idx);
+         }
+      }
+   }
+   return(CT_NONE);
+}
+
 static bool ends_with(const char *filename, const char *tag)
 {
    int len1 = strlen(filename);
@@ -1197,6 +1366,7 @@ struct file_lang languages[] =
    { ".m",    "OC",   LANG_OC            },
    { ".mm",   "OC+",  LANG_OC | LANG_CPP },
    { ".sqc",  "",     LANG_C             }, // embedded SQL
+   { ".es",   "ECMA", LANG_ECMA          },
 };
 
 /**
