@@ -6,7 +6,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: uncrustify_types.h 1234 2008-06-14 22:03:53Z bengardner $
+ * $Id: uncrustify_types.h 1495 2009-02-28 17:37:10Z bengardner $
  */
 
 #ifndef UNCRUSTIFY_TYPES_H_INCLUDED
@@ -64,6 +64,7 @@ struct paren_stack_entry
    c_token_t parent;       /**< if, for, function, etc */
    brstage_e stage;
    bool      in_preproc;   /**< whether this was created in a preprocessor */
+   bool      non_vardef;   /**< Hit a non-vardef line */
 };
 
 /* TODO: put this on a linked list */
@@ -119,7 +120,7 @@ struct parse_frame
 #define PCF_PUNCTUATOR         (1 << 26)
 #define PCF_INSERTED           (1 << 27)  /* chunk was inserted from another file */
 #define PCF_ALIGN_START        (1 << 28)
-#define PCF_TBD_29             (1 << 29)
+#define PCF_VAR_TYPE           (1 << 29)  /* part of a variable def type */
 #define PCF_TBD_30             (1 << 30)
 #define PCF_OWN_STR            (1 << 31)  /* chunk owns the memory at str */
 
@@ -162,7 +163,7 @@ static const char *pcf_names[] =
    "PUNCTUATOR",
    "INSERTED",
    "ALIGN_START",
-   "TBD_29",
+   "VAR_TYPE",
    "TBD_30",
    "OWN_STR",
 };
@@ -170,11 +171,19 @@ static const char *pcf_names[] =
 
 struct align_ptr_t
 {
-   chunk_t *next; /* NULL or the chunk that should be under this one */
-   bool    right_align;
-   int     star_style;
-   int     amp_style;
-   int     gap;
+   chunk_t *next;       /* NULL or the chunk that should be under this one */
+   bool    right_align; /* AlignStack.m_right_align */
+   int     star_style;  /* AlignStack.m_star_style */
+   int     amp_style;   /* AlignStack.m_amp_style */
+   int     gap;         /* AlignStack.m_gap */
+
+   /* col_adj is the amount to alter the column for the token.
+    * For example, a dangling '*' would be set to -1.
+    * A right-aligned word would be a positive value.
+    */
+   int     col_adj;
+   chunk_t *ref;
+   chunk_t *start;
 };
 
 /** This is the main type of this program */
@@ -196,9 +205,9 @@ struct chunk_t
    int         level;            /* nest level in {, (, or [ */
    int         brace_level;      /* nest level in braces only */
    int         pp_level;         /* nest level in #if stuff */
-   bool        after_tab;
-   int         len;
-   const char  *str;
+   bool        after_tab;        /* whether this token was after a tab */
+   int         len;              /* # of bytes at str that make up the token */
+   const char  *str;             /* pointer to the token text */
 };
 
 enum
@@ -211,9 +220,10 @@ enum
    LANG_OC   = 0x0020,     /*<< Objective C */
    LANG_VALA = 0x0040,     /*<< Like C# */
    LANG_PAWN = 0x0080,
+   LANG_ECMA = 0x0100,
 
-   LANG_ALLC = 0x007f,
-   LANG_ALL  = 0x00ff,
+   LANG_ALLC = 0x017f,
+   LANG_ALL  = 0x0fff,
 
    FLAG_PP   = 0x8000,     /*<< only appears in a preprocessor */
 };
@@ -229,6 +239,7 @@ enum pattern_class
    PATCLS_OPBRACED, // keyword + optional parens + braced stmt: catch, version
    PATCLS_VBRACED,  // keyword + value + braced stmt: namespace
    PATCLS_PAREN,    // keyword + parens: while-of-do
+   PATCLS_OPPAREN,  // keyword + optional parens: invariant (D lang)
    PATCLS_ELSE,     // Special case of PATCLS_BRACED for handling CT_IF
 };
 
@@ -291,6 +302,7 @@ struct cp_data
    const char         *filename;
 
    file_mem           file_hdr;  /* for cmt_insert_file_header */
+   file_mem           file_ftr;  /* for cmt_insert_file_footer */
    file_mem           func_hdr;  /* for cmt_insert_func_header */
    file_mem           class_hdr; /* for cmt_insert_class_header */
 
