@@ -3,7 +3,7 @@ int foo(int a, int b)
 {
 if ( ! (a || b) )
 {
-return 1;
+return (1);
 }
 return 0;
 }
@@ -42,6 +42,17 @@ void foo3()
    {
       bar();
    }
-
+  return ;
 }
 
+int foo1(void)
+{
+   FOO_ERROR("connect: can only connected from state CLOSED", pcb->state == CLOSED, return ERR_ISCONN);
+return ERR_OK;
+}
+
+int foo2(void)
+{
+
+return (-1);
+}
