@@ -10,8 +10,7 @@
 int main(int argc, char *argv[])
 {
     int a, b;
-    a = 1; /* stupid comment \\
- */
+    a = 1; /* stupid comment \\ */
     b = 2;
 
     return(a+b);
