@@ -6,7 +6,7 @@
 # This could all be done with bash, but I wanted to use python. =)
 # Anyway, this was all done while waiting in the Denver airport.
 #
-# $Id: run_tests.py 1274 2008-06-21 17:08:55Z bengardner $
+# $Id: run_tests.py 1456 2008-12-13 22:24:43Z bengardner $
 #
 
 import sys
@@ -156,7 +156,7 @@ if __name__ == '__main__':
 			args.append(arg)
 
 	if len(args) == 0:
-		the_tests += "c-sharp c cpp d java pawn objective-c vala".split()
+		the_tests += "c-sharp c cpp d java pawn objective-c vala ecma".split()
 	else:
 		the_tests += args
 
