@@ -6,7 +6,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: windows_compat.h 923 2007-09-18 13:04:22Z bengardner $
+ * $Id: windows_compat.h 1004 2007-11-08 03:34:27Z bengardner $
  */
 #ifndef WINDOWS_COMPAT_H_INCLUDED
 #define WINDOWS_COMPAT_H_INCLUDED
@@ -56,6 +56,12 @@ typedef unsigned int       UINT32;
 #undef strncasecmp
 #define strncasecmp   _strnicmp
 
+#undef strdup
+#define strdup        _strdup
+
+#undef fileno
+#define fileno        _fileno
+
 /* includes for _setmode() */
 #include <io.h>
 #include <fcntl.h>
